@@ -1,6 +1,7 @@
 /* -*- c-file-style: "ruby"; indent-tabs-mode: nil -*- */
 
 /*
+ *  Copyright (C) 2007 Kouhei Sutou <kou@cozmixng.org>
  *  Copyright (C) 2006 Eriko Sato
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -21,7 +22,7 @@
 #include "kz-rb-ext.h"
 
 #include "kz-embed.h"
-#include "kz-mozembed.h"
+#include "kz-embed.h"
 
 #define _SELF(obj) (KZ_EMBED(RVAL2GOBJ(obj)))
 #define RVAL2KZBOOKMARK(obj) (KZ_BOOKMARK(RVAL2GOBJ(obj)))
@@ -33,7 +34,8 @@
 static VALUE
 rb_kz_embed_get_window(VALUE self)
 {
-    return GOBJ2RVAL(kz_window_get_from_tab(RVAL2GOBJ(self)));
+    return GOBJ2RVAL(kz_app_get_window_from_tab(KZ_RB_GET_APP,
+                                                RVAL2GOBJ(self)));
 }
 
 static VALUE
@@ -620,13 +622,13 @@ rb_kz_embed_show_page_certificate(VALUE self)
 }
 
 void
-Init_kz_rb_embed(VALUE mKz)
+_kz_rb_embed_init(VALUE mKz)
 {
     VALUE cKzEmbed;
-    VALUE cKzMozEmbed;
+    VALUE cKzGeckoEmbed;
 
     cKzEmbed = G_DEF_CLASS(KZ_TYPE_EMBED, "Embed", mKz);
-    cKzMozEmbed = G_DEF_CLASS(KZ_TYPE_MOZ_EMBED, "MozEmbed", mKz);
+    cKzGeckoEmbed = G_DEF_CLASS3("KzGeckoEmbed", "GeckoEmbed", mKz);
 
     G_DEF_CLASS(KZ_TYPE_EMBED_LOAD_FLAG, "EmbedLoadFlag", mKz);
     G_DEF_CLASS(KZ_TYPE_EMBED_RELOAD_FLAG, "EmbedReloadFlag", mKz);
