@@ -18,11 +18,17 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif /* HAVE_CONFIG_H */
+
 #include "kz-actions-popup.h"
 
 #include <string.h>
-#include <unistd.h>
 #include <glib/gi18n.h>
+#ifdef HAVE_UNISTD_H
+#  include <unistd.h>
+#endif
 #include "kz-entry-action.h"
 #include "gtk-utils.h"
 #include "glib-utils.h"
@@ -35,6 +41,7 @@
 #include "kz-xml.h"
 #include "kz-actions-dynamic.h"
 #include "kz-icons.h"
+#include "kz-notebook.h"
 
 #define KZ_ACTIONS_POPUP_LANGUAGE_KEY "KzActionsPopup::Language"
 #define KZ_ACTIONS_POPUP_TAB_KEY "KzActionsPopup::Tab"
@@ -101,7 +108,7 @@ open_location (GtkAction *action, KzWindow *kz, LocationType location, WindowTyp
 		break;
 	case NEW_WINDOW:
 	{
-		GtkWidget *widget = kz_window_new(uri);
+		GtkWidget *widget = kz_app_create_new_window(kz_app, uri);
 		gtk_widget_show(widget);
 		break;
 	}
@@ -375,10 +382,10 @@ cb_tablist_menuitem_activate (GtkWidget *menuitem, KzWindow *kz)
 	GtkWidget *widget = g_object_get_data(G_OBJECT(menuitem),
 					      KZ_ACTIONS_POPUP_TAB_KEY);
 
-	page_num = gtk_notebook_page_num(GTK_NOTEBOOK(kz->notebook),
-					 GTK_WIDGET(widget));
+	page_num = kz_notebook_page_num(KZ_NOTEBOOK(kz->notebook),
+					GTK_WIDGET(widget));
 
-	gtk_notebook_set_current_page(GTK_NOTEBOOK(kz->notebook), page_num);
+	kz_notebook_set_current_page(KZ_NOTEBOOK(kz->notebook), page_num);
 }
 
 
@@ -414,7 +421,7 @@ act_popup_tab_list (GtkAction *action, KzWindow *kz)
 					     G_CALLBACK(cb_popup_menu_hide), kz);
 }
 
-GtkActionEntry kz_actions_popup[] =
+static GtkActionEntry kz_actions_popup[] =
 {
         /* Toplevel */
 
@@ -453,7 +460,7 @@ GtkActionEntry kz_actions_popup[] =
 	{"StockOpenSmartBookmark",      NULL, N_("Open Smartbookmark by selected text"),  NULL, NULL, NULL},
 };
 
-const gint kz_actions_popup_len = G_N_ELEMENTS(kz_actions_popup);
+static const gint kz_actions_popup_len = G_N_ELEMENTS(kz_actions_popup);
 
 
 GtkActionGroup *
@@ -602,6 +609,7 @@ kz_actions_popup_append_encoding_menuitem (KzWindow *kz,
 	GtkWidget *menuitem;
 	GSList *group = NULL;
 	GtkWidget *page = KZ_WINDOW_CURRENT_PAGE(kz);
+	gchar *encodings_xml;
 	gchar *current_encoding = NULL;
 	gboolean forced = FALSE;
 	KzXML *xml;
@@ -618,7 +626,7 @@ kz_actions_popup_append_encoding_menuitem (KzWindow *kz,
 
 	/* append encodings menuitem */
 	encoding_submenu = gtk_menu_new();
-		
+
 	/* get current charset */
 	if(KZ_EMBED(page))
 	{
@@ -648,7 +656,12 @@ kz_actions_popup_append_encoding_menuitem (KzWindow *kz,
 	gtk_widget_show(menuitem);
 
 	xml = kz_xml_new();
-	kz_xml_load(xml, KZ_SYSCONFDIR "/mozilla/encodings.xml");
+	encodings_xml = g_build_filename(KZ_GET_SYSTEM_CONFIG_DIR,
+					 "mozilla",
+					 "encodings.xml",
+					 NULL);
+	kz_xml_load(xml, encodings_xml);
+	g_free(encodings_xml);
 	node = kz_xml_get_root_element(xml);
 	if (node && kz_xml_node_name_is(node, "encodings"))
 	{
@@ -683,12 +696,12 @@ static void
 kz_actions_popup_append_tablist_menuitem (KzWindow *kz, GtkWidget *tablist_menu)
 {
 	gint i;
-	const gint num = gtk_notebook_get_n_pages(GTK_NOTEBOOK(kz->notebook));
 	GtkWidget *tablist_submenu;
 	GtkWidget *page;
 	gint current_page_num;
-	KzFavicon *kzfav = kz->kzfav;
+	KzFavicon *kzfav = KZ_GET_FAVICON;
 	gchar *color = NULL;
+	const gint num = kz_notebook_get_n_pages(KZ_NOTEBOOK(kz->notebook));
 
 	/* label color */
 	color = KZ_CONF_GET_STR("Tab" , "normal_color");
@@ -740,8 +753,8 @@ kz_actions_popup_append_tablist_menuitem (KzWindow *kz, GtkWidget *tablist_menu)
 	}
 
 	page = KZ_WINDOW_CURRENT_PAGE(kz);
-	current_page_num = gtk_notebook_page_num(GTK_NOTEBOOK(kz->notebook),
-						 GTK_WIDGET(page));
+	current_page_num = kz_notebook_page_num(KZ_NOTEBOOK(kz->notebook),
+						GTK_WIDGET(page));
 
 	for (i = 0; i < num; i++)
 	{
