@@ -55,10 +55,12 @@
 #include "kz-bookmark-menu.h"
 #include "kz-autoscroller.h"
 #include "kz-popup-tablist.h"
-#include "eggregex.h"
 #include "kz-proxy-folder.h"
 #include "kz-icons.h"
 #include "kz-statusbar.h"
+#include "kz-notebook.h"
+#include "kz-session.h"
+#include "glib-utils.h"
 
 #include <time.h>
 
@@ -103,9 +105,9 @@ act_close_window (GtkAction *action, KzWindow *kz)
 {
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
-	kz_window_store_state(kz);
-	kz_window_close_all_tab(kz);
+	kz_session_freeze(KZ_SESSION(KZ_GET_CURRENT_SESSION));
 	gtk_widget_destroy(GTK_WIDGET(kz));
+	kz_session_thaw(KZ_SESSION(KZ_GET_CURRENT_SESSION));
 }
 
 
@@ -125,7 +127,7 @@ act_new_window (GtkAction *action, KzWindow *kz)
 
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
-	widget = kz_window_new(NULL);
+	widget = kz_app_create_new_window(kz_app, NULL);
 	gtk_widget_show(widget);
 }
 
@@ -166,11 +168,11 @@ act_new_tab (GtkAction *action, KzWindow *kz)
    	KZ_CONF_GET("Tab", "focus_open_new", focus, BOOL);
 	if(focus && widget)
 	{
-		int num = gtk_notebook_page_num(GTK_NOTEBOOK(kz->notebook),
-						widget);
+		int num = kz_notebook_page_num(KZ_NOTEBOOK(kz->notebook),
+					       widget);
 		if(num != -1)
-			gtk_notebook_set_current_page(GTK_NOTEBOOK(kz->notebook),
-						      num);
+			kz_notebook_set_current_page(KZ_NOTEBOOK(kz->notebook),
+						     num);
 
 		KZ_CONF_GET("Tab","focus_loc_ent_new", focus, BOOL);
 		if(focus)
@@ -206,11 +208,10 @@ act_focus_loc_ent (GtkAction *action, KzWindow *kz)
 static void
 act_clear_loc_ent (GtkAction *action, KzWindow *kz)
 {
-	GtkAction *loc_ent = gtk_action_group_get_action(kz->actions, "LocationEntry");
 	GtkAction *focus_action  = gtk_action_group_get_action(kz->actions,
 							       "FocusLocationEntry");
 
-	kz_entry_action_set_text(KZ_ENTRY_ACTION(loc_ent), "");
+	kz_window_set_location_entry_text(kz, NULL);
 	gtk_action_activate(focus_action);
 }
 
@@ -244,16 +245,7 @@ act_open_file (GtkAction *action, KzWindow *kz)
 
 	if (filename)
         {
-		gchar *utf8, *urlencode, *dirname, *basename;
-		basename = g_path_get_basename(filename);
-		dirname = g_path_get_dirname(filename);
-		utf8 = g_filename_to_utf8 (basename, -1, NULL, NULL, NULL);
-		urlencode = url_encode(utf8);
-		uri = g_strdup_printf("file://%s/%s", dirname, urlencode);
-                g_free(utf8);
-                g_free(basename);
-                g_free(dirname);
-		g_free(urlencode);
+		uri = g_filename_to_uri(filename, NULL, NULL);
 		g_free(filename);
         }
 
@@ -304,17 +296,18 @@ act_open_selected_text (GtkAction *action, KzWindow *kz)
 {
 	GtkWidget *widget = KZ_WINDOW_CURRENT_PAGE(kz);
 	gchar *text;
-	EggRegex *egg_regex;
-	gint n_match, i = 1;
+	GRegex *regex;
+	gint i = 1;
+	gboolean match;
 	GError *error = NULL;
 
  	g_return_if_fail (KZ_IS_WINDOW((kz)));
  
 	if (!KZ_IS_EMBED(widget)) return;
 	
-	egg_regex = egg_regex_new(HTTP_REGEX, 
-				  0,
-				  0, &error);
+	regex = g_regex_new(HTTP_REGEX, 
+			    0,
+			    0, &error);
 
 	if (error)
 	{
@@ -325,12 +318,12 @@ act_open_selected_text (GtkAction *action, KzWindow *kz)
 	text = kz_embed_get_selection_string(KZ_EMBED(widget));
 	if (!text) return;
 
-	n_match = egg_regex_match(egg_regex, text, -1, 0);
+	match = g_regex_match(regex, text, 0);
 	
-	if (n_match > 0)
+	if (match)
 	{
 		gchar **uri;
-		uri = egg_regex_fetch_all (egg_regex, text);
+		uri = g_regex_fetch_all(regex, text);
 
 		while (uri[i])
 		{
@@ -343,7 +336,7 @@ act_open_selected_text (GtkAction *action, KzWindow *kz)
 			g_strfreev(uri);
 	}
 
-	egg_regex_free(egg_regex);
+	g_regex_free(regex);
 	g_free(text);
 }
 
@@ -519,7 +512,7 @@ act_stop_all (GtkAction *action, KzWindow *kz)
 
 	g_return_if_fail(KZ_IS_WINDOW (kz));
 
-	num = gtk_notebook_get_n_pages(GTK_NOTEBOOK(kz->notebook));
+	num = kz_notebook_get_n_pages(KZ_NOTEBOOK(kz->notebook));
 
 	for (i = num - 1; i >= 0; i--)
 	{
@@ -541,7 +534,7 @@ act_reload_all  (GtkAction *action, KzWindow *kz)
 
 	gdk_window_get_pointer(NULL, &x, &y, &state);
 
-	num = gtk_notebook_get_n_pages(GTK_NOTEBOOK(kz->notebook));
+	num = kz_notebook_get_n_pages(KZ_NOTEBOOK(kz->notebook));
 
 	for (i = num - 1; i >= 0; i--)
 	{
@@ -569,9 +562,8 @@ act_view_source (GtkAction *action, KzWindow *kz)
 		newtab = kz_window_open_new_tab(kz, NULL);
 
 		kz_embed_copy_page(KZ_EMBED(widget),
-				   KZ_EMBED(newtab));
-		kz_embed_view_source(KZ_EMBED(newtab),
-				     kz_embed_get_location(KZ_EMBED(widget)));
+				   KZ_EMBED(newtab),
+				   KZ_EMBED_COPY_SOURCE);
 	}
 }
 
@@ -636,47 +628,18 @@ act_find (GtkAction *action, KzWindow *kz)
 static void
 act_prev_tab (GtkAction *action, KzWindow *kz)
 {
-	GtkNotebook *notebook;
-	gboolean circulation = FALSE;
-
-	notebook = GTK_NOTEBOOK(kz->notebook);
 	g_return_if_fail(KZ_IS_WINDOW(kz));
-	KZ_CONF_GET("Tab", "wheel_circulation", circulation, BOOL);
 
-	if (circulation &&
-	    gtk_notebook_get_current_page(notebook) == 0)
-	{
-		gtk_notebook_set_current_page
-			(notebook, gtk_notebook_get_n_pages(notebook) - 1);
-	}
-	else
-	{
-		gtk_notebook_prev_page(notebook);
-	}
+	kz_notebook_prev_tab(KZ_NOTEBOOK(kz->notebook));
 }
 
 
 static void
 act_next_tab (GtkAction *action, KzWindow *kz)
 {
-	GtkNotebook *notebook;
-	gboolean circulation = FALSE;
-
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
-	notebook = GTK_NOTEBOOK(kz->notebook);
-	KZ_CONF_GET("Tab", "wheel_circulation", circulation, BOOL);
-	
-	if (circulation && 
-	    gtk_notebook_get_current_page(notebook) == 
-	    gtk_notebook_get_n_pages(notebook) - 1)
-	{
-		gtk_notebook_set_current_page(notebook, 0);
-	}
-	else
-	{
-		gtk_notebook_next_page(notebook);
-	}
+	kz_notebook_next_tab(KZ_NOTEBOOK(kz->notebook));
 }
 
 
@@ -703,8 +666,8 @@ act_detaild_preference (GtkAction *action, KzWindow *kz)
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
 	widget = kz_window_open_new_tab(kz, "about:config");
-	num = gtk_notebook_page_num(GTK_NOTEBOOK(kz->notebook), widget);
-	gtk_notebook_set_current_page(GTK_NOTEBOOK(kz->notebook), num);
+	num = kz_notebook_page_num(KZ_NOTEBOOK(kz->notebook), widget);
+	kz_notebook_set_current_page(KZ_NOTEBOOK(kz->notebook), num);
 }
 
 
@@ -938,61 +901,40 @@ act_tab_pos (GtkRadioAction *action, GtkRadioAction *cur_action, KzWindow *kz)
 }
 
 
+static void
+close_tabs (KzNotebook *notebook, KzNotebookCloseCondition condition)
+{
+	gint pos;
+
+	pos = kz_notebook_get_current_page(notebook);
+
+	kz_notebook_close_tabs(notebook, condition, pos);
+}
+
 static void
 act_tab_close_backward_all (GtkAction *action, KzWindow *kz)
 {
-	gint i, pos, num;
-
 	g_return_if_fail(KZ_IS_WINDOW (kz));
 
-	pos = gtk_notebook_get_current_page(GTK_NOTEBOOK(kz->notebook));
-	num = gtk_notebook_get_n_pages(GTK_NOTEBOOK(kz->notebook));
-
-	for (i = pos - 1; i >= 0; i--)
-	{
-		GtkWidget *widget = KZ_WINDOW_NTH_PAGE(kz, i);
-
-		kz_window_close_tab(kz, widget);
-	}
+	close_tabs(KZ_NOTEBOOK(kz->notebook), KZ_NOTEBOOK_CLOSE_BACKWARD);
 }
 
 
 static void
 act_tab_close_forward_all (GtkAction *action, KzWindow *kz)
 {
-	gint i, pos, num;
-
 	g_return_if_fail(KZ_IS_WINDOW (kz));
 
-	pos = gtk_notebook_get_current_page(GTK_NOTEBOOK(kz->notebook));
-	num = gtk_notebook_get_n_pages(GTK_NOTEBOOK(kz->notebook));
-
-	for (i = num - 1; i > pos; i--)
-	{
-		GtkWidget *widget = KZ_WINDOW_NTH_PAGE(kz, i);
-
-		kz_window_close_tab(kz, widget);
-	}
+	close_tabs(KZ_NOTEBOOK(kz->notebook), KZ_NOTEBOOK_CLOSE_FORWARD);
 }
 
 
 static void
 act_tab_close_all_inactive_active (GtkAction *action, KzWindow *kz)
 {
-	gint i, pos, num;
-
 	g_return_if_fail(KZ_IS_WINDOW (kz));
 
-	pos = gtk_notebook_get_current_page(GTK_NOTEBOOK(kz->notebook));
-	num = gtk_notebook_get_n_pages(GTK_NOTEBOOK(kz->notebook));
-
-	for (i = num - 1; i >= 0; i--)
-	{
-		GtkWidget *widget = KZ_WINDOW_NTH_PAGE(kz, i);
-
-		if (i == pos) continue;
-		kz_window_close_tab(kz, widget);
-	}
+	close_tabs(KZ_NOTEBOOK(kz->notebook), KZ_NOTEBOOK_CLOSE_INACTIVE);
 }
 
 static void
@@ -1253,7 +1195,7 @@ act_edit_bookmark_bars (GtkAction *action, KzWindow *kz)
 
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
-	bookmark = kz_bookmarks->bookmark_bars;
+	bookmark = KZ_GET_BAR_BOOKMARK;
 	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
 	g_return_if_fail(kz_bookmark_is_folder(bookmark));
 
@@ -1271,7 +1213,7 @@ act_edit_smart_bookmarks (GtkAction *action, KzWindow *kz)
 
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
-	bookmark = kz_bookmarks->smarts;
+	bookmark = KZ_GET_SMART_BOOKMARK;
 	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
 	g_return_if_fail(kz_bookmark_is_folder(bookmark));
 
@@ -1298,7 +1240,7 @@ act_update_bookmark (GtkAction *action, KzWindow *kz)
 static void
 act_clip_selection (GtkAction *action, KzWindow *kz)
 {
-	KzBookmark *clip = kz_bookmarks->clip, *bookmark;
+	KzBookmark *clip, *bookmark;
 	GtkWidget *embed;
 	const gchar *title, *uri;
 	gchar *source = NULL;
@@ -1306,7 +1248,8 @@ act_clip_selection (GtkAction *action, KzWindow *kz)
 	struct tm *date = NULL;
 	time_t t;
 	gchar time_string[20];
-	gchar *filename, *htmlname = NULL, *dirname, *link, *name;
+	gchar *filename, *htmlname = NULL, *link, *name;
+	gchar *dir_name, *clip_dir_name;
 	gchar *pos, *full_htmlname;
 	GIOChannel *io;
 
@@ -1315,6 +1258,7 @@ act_clip_selection (GtkAction *action, KzWindow *kz)
 	embed = KZ_WINDOW_CURRENT_PAGE(kz);
 	if (!KZ_IS_EMBED(embed)) return;
 
+	clip = KZ_GET_CLIP_BOOKMARK;
 	g_get_current_time(&now);
 
 	title = kz_window_get_title(kz);
@@ -1343,15 +1287,13 @@ act_clip_selection (GtkAction *action, KzWindow *kz)
 
 	full_htmlname = g_strconcat(htmlname, ".html", NULL);
 
-	dirname = g_strconcat("clips/", htmlname, time_string, NULL);
-	make_dir(dirname);
+	dir_name = g_strconcat(htmlname, time_string, NULL);
+	clip_dir_name = g_build_filename(KZ_GET_CLIPS_DIR, dir_name, NULL);
+	g_mkdir_with_parents(clip_dir_name, 0700);
 
-	source  = kz_embed_get_html_with_contents(KZ_EMBED(embed),
-						  dirname);
+	source  = kz_embed_get_html_with_contents(KZ_EMBED(embed), dir_name);
 
-	filename = g_build_filename(g_get_home_dir(),
-				    "."PACKAGE,
-				    dirname,
+	filename = g_build_filename(clip_dir_name,
 				    full_htmlname,
 			       	    NULL);
 	
@@ -1372,11 +1314,12 @@ act_clip_selection (GtkAction *action, KzWindow *kz)
 	kz_bookmark_file_set_location(KZ_BOOKMARK_FILE(bookmark), uri);
 	
 	kz_bookmark_prepend(clip, bookmark);
-	kz_bookmark_file_save(KZ_BOOKMARK_FILE(kz_bookmarks->clip));
+	kz_bookmark_file_save(KZ_BOOKMARK_FILE(clip));
 	
 	g_free(link);
 	g_free(filename);
-	g_free(dirname);
+	g_free(dir_name);
+	g_free(clip_dir_name);
 	g_free(htmlname);
 	g_free(full_htmlname);
 }
@@ -1389,8 +1332,7 @@ act_extract_links (GtkAction *action, KzWindow *kz)
 
 	if (!KZ_IS_EMBED(widget)) return;
 
-	dialog = kz_links_dialog_new(kz, KZ_EMBED(widget), FALSE);
-	gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(kz));
+	dialog = kz_links_dialog_new(GTK_WINDOW(kz), KZ_EMBED(widget), FALSE);
 	gtk_widget_show(dialog);
 }
 
@@ -1403,130 +1345,22 @@ act_extract_selected_links (GtkAction *action, KzWindow *kz)
 
 	if (!KZ_IS_EMBED(widget)) return;
 
-	dialog = kz_links_dialog_new(kz, KZ_EMBED(widget), TRUE);
-	gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(kz));
+	dialog = kz_links_dialog_new(GTK_WINDOW(kz), KZ_EMBED(widget), TRUE);
 	gtk_widget_show(dialog);
 }
 
 
-#warning these codes about session will be obsolete. we need changing session function.
 static void
 act_save_session (GtkAction *action, KzWindow *kz)
 {
-	/*FIXME! set current tab position */ 
-	kz_bookmark_file_save(KZ_BOOKMARK_FILE(kz_bookmarks->current_session));
+	kz_app_save_session(kz_app);
 }
 
 
 static void
 act_restore_session (GtkAction *action, KzWindow *kz)
 {
-	GList *list;
-	const GList *node;
-	gchar *current_session;
-	gint i, num, pos = 0;
-
-	g_return_if_fail(KZ_IS_WINDOW (kz));
-
-	if (is_restoring_session) return;
-#if 1
-	{
-	const GList *window_list, *window_node;
-	KzBookmark *tmp_bookmark;
-
-	/* reload from current session file */
-	current_session  = g_build_filename(g_get_home_dir(),
-					    "."PACKAGE,
-					    "current_session.xml",
-					     NULL);
-	if (!g_file_test(current_session, G_FILE_TEST_EXISTS))
-		return;
-	tmp_bookmark = KZ_BOOKMARK(kz_bookmark_file_new(current_session,
-					   _("Current Session"),
-					   "XBEL"));
-	g_free(current_session);
-	kz_bookmark_file_load(KZ_BOOKMARK_FILE(tmp_bookmark));
-
-	window_list = kz_window_get_window_list();
-	for (node = window_list; node; node = g_list_next (node))
-	{
-		KzWindow *window = KZ_WINDOW(node->data);
-		num = gtk_notebook_get_n_pages(GTK_NOTEBOOK(window->notebook));
-		for (i = num - 1; i >= 0; i--)
-		{
-			GtkWidget *widget = KZ_WINDOW_NTH_PAGE(window, i);
-			kz_window_close_tab(window, widget);
-		}
-	}
-
-	list = kz_bookmark_get_children(tmp_bookmark);
-
-	window_node = window_list;
-	/* windows */
-	for (node = list; node; node = g_list_next(node))
-	{
-		GList *tabs, *tabnode;
-		KzWindow *window;
-		KzBookmark *window_bookmark = KZ_BOOKMARK(node->data);
-		tabs = kz_bookmark_get_children(window_bookmark);
-
-		if (!window_node)
-		{
-			window = KZ_WINDOW(kz_window_new(NULL));
-			gtk_widget_show(GTK_WIDGET(window));
-		}
-		else
-			window = KZ_WINDOW(window_node->data);
-	
-		/* tabs */
-		for (tabnode = tabs; tabnode; tabnode = g_list_next(tabnode))
-		{
-			GtkWidget *widget;
-			KzBookmark *child = KZ_BOOKMARK(tabnode->data);
-			KzTabLabel *kztab;
-			widget = kz_window_open_new_tab_at_tail(window, NULL);
-			kz_embed_set_history(KZ_EMBED(widget), child);
-			kztab = KZ_TAB_LABEL(gtk_notebook_get_tab_label(GTK_NOTEBOOK(window->notebook),
-									widget));
-
-			if (kz_bookmark_get_lock(child))
-			{
-				kz_tab_label_set_lock(kztab, TRUE);
-			}
-			if (kz_bookmark_get_auto_refresh(child))
-			{
-				kz_tab_label_set_auto_refresh(kztab, TRUE);
-			}
-			if (kz_bookmark_get_javascript(child))
-			{
-				kz_embed_set_allow_javascript(KZ_EMBED(kztab->kzembed), TRUE);
-				kz_tab_label_set_javascript(kztab, TRUE);
-			}
-			else
-			{
-				kz_embed_set_allow_javascript(KZ_EMBED(kztab->kzembed), FALSE);
-				kz_tab_label_set_javascript(kztab, FALSE);
-			}
-		}
-		g_list_free(tabs);
-		pos = kz_bookmark_get_current(window_bookmark);
-		gtk_notebook_set_current_page(GTK_NOTEBOOK(window->notebook), pos);
-
-		if (window_node)
-			window_node = g_list_next(window_node);
-	}
-	g_list_free(list);
-
-	for (;window_node; window_node = g_list_next(window_node))
-	{
-		KzWindow *window = KZ_WINDOW(window_node->data);
-		gtk_widget_destroy(GTK_WIDGET(window));
-	}
-	
-	kz_bookmark_file_save(KZ_BOOKMARK_FILE(tmp_bookmark));
-	g_object_unref(tmp_bookmark);
-	}
-#endif
+	kz_app_restore_session(kz_app);
 }
 
 
@@ -1534,7 +1368,7 @@ static void
 act_about (GtkAction *action, KzWindow *kz)
 {
 	g_return_if_fail(KZ_IS_WINDOW(kz));
-	kz_about_dialog_new(kz);
+	kz_about_dialog_new(GTK_WINDOW(kz));
 }
 
 static void
@@ -1546,8 +1380,8 @@ act_open_kazehakase_website(GtkAction *action, KzWindow *kz)
 	g_return_if_fail(KZ_IS_WINDOW(kz));
 
 	widget = kz_window_open_new_tab(kz, KAZEHAKASE_URI);
-	num = gtk_notebook_page_num(GTK_NOTEBOOK(kz->notebook), widget);
-	gtk_notebook_set_current_page(GTK_NOTEBOOK(kz->notebook), num);
+	num = kz_notebook_page_num(KZ_NOTEBOOK(kz->notebook), widget);
+	kz_notebook_set_current_page(KZ_NOTEBOOK(kz->notebook), num);
 }
 
 static void
@@ -1562,7 +1396,7 @@ act_copy_tab (GtkAction *action, KzWindow *kz)
 	kzembed  = KZ_EMBED(widget);	
 	dkzembed = KZ_EMBED(kz_window_open_new_tab(kz, NULL));
 
-	kz_embed_copy_page(kzembed, dkzembed);
+	kz_embed_copy_page(kzembed, dkzembed, KZ_EMBED_COPY_NORMAL);
 	kz_embed_shistory_copy(kzembed, dkzembed, TRUE, TRUE, TRUE);
 }
 
@@ -1611,8 +1445,7 @@ act_show_password_manager (GtkAction *action, KzWindow *kz)
 {
 	GtkWidget *dialog;
 
-	dialog = kz_password_manager_dialog_new(kz);
-	gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(kz));
+	dialog = kz_password_manager_dialog_new(GTK_WINDOW(kz));
 	gtk_widget_show(dialog);
 }
 
@@ -1621,8 +1454,7 @@ act_show_copy_format(GtkAction *action, KzWindow *kz)
 {
 	GtkWidget *dialog;
 
-	dialog = kz_copy_format_dialog_new(kz);
-	gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(kz));
+	dialog = kz_copy_format_dialog_new(GTK_WINDOW(kz));
 	gtk_widget_show(dialog);
 }
 
@@ -1811,7 +1643,7 @@ cb_thumbnail_tablist_selected (KzPopupTablist *popup,
 			       KzBookmark* tab, gint index,
 			       KzWindow* kz)
 {
-	gtk_notebook_set_current_page(GTK_NOTEBOOK(kz->notebook), index);
+	kz_notebook_set_current_page(KZ_NOTEBOOK(kz->notebook), index);
 
 	gtk_widget_destroy(GTK_WIDGET(popup));
 	kz->popup_tablist = NULL;
@@ -1884,10 +1716,7 @@ act_show_hide_bookmarkbars (GtkAction *action, KzWindow *kz)
 	g_return_if_fail(GTK_IS_WIDGET(kz->bookmark_bars_area));
 
 	active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION(action));
-	if (active)
-		gtk_widget_show(kz->bookmark_bars_area);
-	else
-		gtk_widget_hide(kz->bookmark_bars_area);
+	g_object_set(kz->bookmark_bars_area, "visible", active, NULL);
 }
 
 static void
@@ -2011,7 +1840,7 @@ GtkActionEntry kz_actions[] =
 
   {"ViewPageSource", KZ_STOCK_VIEW_SOURCE, N_("View page source"), NULL, N_("View the source code of the page"), G_CALLBACK(act_view_source)},
   {"ShowPageCertificate", NULL, N_("Show page certificate"), NULL, N_("Show the certificate of the page"), G_CALLBACK(act_show_page_certificate)},
-  {"AutoScrollMode", NULL,          N_("Autoscroll mode"), NULL, N_("set auto-scroll mode"), G_CALLBACK(act_auto_scroll_mode)},
+  {"AutoScrollMode", NULL,          N_("Autoscroll mode"), NULL, N_("Set auto-scroll mode"), G_CALLBACK(act_auto_scroll_mode)},
   {"ThumbnailTabList", NULL,          N_("ThumbnailTabList"), NULL, N_("Display the tab list with thumbnail"), G_CALLBACK(act_thumbnail_tablist)},
 /*
   {"EditMode", KZ_STOCK_VIEW_SOURCE, N_("EditMode"), NULL, N_("View the source code of the page"), G_CALLBACK(act_set_edit_mode)},
@@ -2169,7 +1998,7 @@ combine_favicon(GtkAction *action, const gchar *uri, KzWindow *kz)
 {
 	GtkWidget *widget;
 	GSList *node, *proxies;
-	KzFavicon *kzfav = kz->kzfav;
+	KzFavicon *kzfav = KZ_GET_FAVICON;
 
 	proxies = gtk_action_get_proxies(action);
 	for (node = proxies; node; node = g_slist_next(node))
@@ -2277,12 +2106,12 @@ kz_actions_set_tab_sensitive (KzWindow *kz, KzEmbed *kzembed)
 	GtkAction *action;
 	g_return_if_fail (KZ_IS_WINDOW(kz));
 
-	num = gtk_notebook_get_n_pages(GTK_NOTEBOOK(kz->notebook));
+	num = kz_notebook_get_n_pages(KZ_NOTEBOOK(kz->notebook));
 	
 	if (!num) return;
 
-	pos = gtk_notebook_page_num(GTK_NOTEBOOK(kz->notebook),
-				    GTK_WIDGET(kzembed));
+	pos = kz_notebook_page_num(KZ_NOTEBOOK(kz->notebook),
+				   GTK_WIDGET(kzembed));
 	if (num > 1)
 	{
 		action = gtk_action_group_get_action(kz->actions,
@@ -2357,7 +2186,7 @@ kz_actions_set_sensitive (KzWindow *kz, KzEmbed *kzembed)
 	g_return_if_fail (KZ_IS_WINDOW(kz));
 	/* g_return_if_fail (KZ_IS_EMBED (kzembed)); */
 
-	num = gtk_notebook_get_n_pages(GTK_NOTEBOOK(kz->notebook));
+	num = kz_notebook_get_n_pages(KZ_NOTEBOOK(kz->notebook));
 
 	if (kzembed)
 	{
@@ -2376,8 +2205,8 @@ kz_actions_set_sensitive (KzWindow *kz, KzEmbed *kzembed)
 		
 		can_close_tab = !kz_tab_label_get_lock(kztab);
 
-		pos = gtk_notebook_page_num(GTK_NOTEBOOK(kz->notebook),
-					    GTK_WIDGET(kzembed));
+		pos = kz_notebook_page_num(KZ_NOTEBOOK(kz->notebook),
+					   GTK_WIDGET(kzembed));
 		for (i = 0; i< num; i++)
 		{
 			GtkWidget *widget;
@@ -2519,7 +2348,7 @@ kz_actions_set_sensitive (KzWindow *kz, KzEmbed *kzembed)
 	active = gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action));
 	KZ_WINDOW_SET_VISIBLE(kz, "StockSidebarPosMenu", active);
 
-	set_popup_menu_sensitive(kz, kz_bookmarks->menu);
+	set_popup_menu_sensitive(kz, KZ_GET_MENU_BOOKMARK);
 }
 
 
@@ -2612,7 +2441,7 @@ kz_actions_get_bookmark_for_action (KzWindow *kz)
 	if (KZ_IS_BOOKMARK(bookmark))
 		return bookmark;
 
-	return kz_bookmarks->menu;
+	return KZ_GET_MENU_BOOKMARK;
 }
 
 
@@ -2670,7 +2499,7 @@ kz_actions_popup_bookmark_menu_modal (KzWindow *kz, KzBookmark *bookmark,
 
 	kz_actions_set_bookmark_for_action (kz, NULL);
 
-	set_popup_menu_sensitive (kz, kz_bookmarks->menu);
+	set_popup_menu_sensitive (kz, KZ_GET_MENU_BOOKMARK);
 }
 
 
