@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark-menu-item.c,v 1.8 2006/02/16 14:47:32 furyo Exp $
+ *  $Id: kz-bookmark-menu-item.c 2854 2007-03-14 03:22:32Z ikezoe $
  */
 
 #include "kz-bookmark-menu-item.h"
@@ -26,29 +26,19 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
-#include "gobject-utils.h"
 #include "kazehakase.h"
 #include "kz-icons.h"
 #include "kz-actions.h"
 
-static void     kz_bookmark_menu_item_class_init       (KzBookmarkMenuItemClass *klass);
-static void     kz_bookmark_menu_item_init             (KzBookmarkMenuItem *menu);
-
 /* object class methods */
-static GObject* kz_bookmark_menu_item_constructor      (GType           type,
-						        guint           n_props,
-						        GObjectConstructParam *props);
-static void     kz_bookmark_menu_item_dispose          (GObject        *object);
+static GObject *constructor      (GType           type,
+                                  guint           n_props,
+                                  GObjectConstructParam *props);
 
-static gboolean kz_bookmark_menu_item_button_release (GtkWidget *widget, GdkEventButton *event);
-static void     kz_bookmark_menu_item_activate       (GtkMenuItem *item);
+static gboolean button_release (GtkWidget *widget, GdkEventButton *event);
+static void     activate       (GtkMenuItem *item);
 
-static KzBookmarkBaseMenuItemClass *parent_class = NULL;
-
-
-KZ_OBJECT_GET_TYPE(kz_bookmark_menu_item, "KzBookmarkMenuItem", KzBookmarkMenuItem,
-		   kz_bookmark_menu_item_class_init, kz_bookmark_menu_item_init,
-		   KZ_TYPE_BOOKMARK_BASE_MENU_ITEM)
+G_DEFINE_TYPE(KzBookmarkMenuItem, kz_bookmark_menu_item, KZ_TYPE_BOOKMARK_BASE_MENU_ITEM)
 
 static void
 kz_bookmark_menu_item_class_init (KzBookmarkMenuItemClass *klass)
@@ -57,19 +47,16 @@ kz_bookmark_menu_item_class_init (KzBookmarkMenuItemClass *klass)
 	GtkWidgetClass *widget_class;
 	GtkMenuItemClass *menu_class;
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	gobject_class   = (GObjectClass *) klass;
 	widget_class    = (GtkWidgetClass *) klass;
 	menu_class      = (GtkMenuItemClass *) klass;
 
 	/* GObject signals */
-	gobject_class->dispose      = kz_bookmark_menu_item_dispose;
-	gobject_class->constructor  = kz_bookmark_menu_item_constructor;
+	gobject_class->constructor  = constructor;
 
-	widget_class->button_release_event = kz_bookmark_menu_item_button_release;
+	widget_class->button_release_event = button_release;
 
-	menu_class->activate = kz_bookmark_menu_item_activate;
+	menu_class->activate = activate;
 }
 
 
@@ -81,9 +68,9 @@ kz_bookmark_menu_item_init (KzBookmarkMenuItem *menu)
 
 
 static GObject*
-kz_bookmark_menu_item_constructor (GType                  type,
-			      guint                  n_props,
-			      GObjectConstructParam *props)
+constructor (GType                  type,
+             guint                  n_props,
+             GObjectConstructParam *props)
 {
 	KzBookmarkMenuItem *item;
 	KzBookmarkBaseMenuItem *parent;
@@ -94,7 +81,7 @@ kz_bookmark_menu_item_constructor (GType                  type,
 	GtkWidget *accel_label;	
 	KzFavicon *kzfav;
 	KzBookmark *bookmark;
-	GObjectClass *klass = G_OBJECT_CLASS(parent_class);
+	GObjectClass *klass = G_OBJECT_CLASS(kz_bookmark_menu_item_parent_class);
 
 	object = klass->constructor(type, n_props, props);
 	item = KZ_BOOKMARK_MENU_ITEM(object);
@@ -136,12 +123,10 @@ kz_bookmark_menu_item_constructor (GType                  type,
 #endif
 
 	/* favicon  */
-	/* kzfav = parent->kz->kzfav; */
-	kzfav = kz_favicon_get_instance();
+	kzfav = KZ_GET_FAVICON;
 	image = kz_favicon_get_widget(kzfav,
 			kz_bookmark_get_link(bookmark),
 			KZ_ICON_SIZE_BOOKMARK_MENU);
-	g_object_unref(kzfav);
 	if (image)
 	{
 		gtk_widget_show(image);
@@ -153,15 +138,6 @@ kz_bookmark_menu_item_constructor (GType                  type,
 }
 
 
-static void
-kz_bookmark_menu_item_dispose (GObject *object)
-{
-	if (G_OBJECT_CLASS (parent_class)->dispose)
-		G_OBJECT_CLASS (parent_class)->dispose(object);
-}
-
-
-
 GtkWidget *
 kz_bookmark_menu_item_new (KzWindow *kz, KzBookmark *bookmark)
 {
@@ -180,13 +156,13 @@ kz_bookmark_menu_item_new (KzWindow *kz, KzBookmark *bookmark)
 
 
 static gboolean
-kz_bookmark_menu_item_button_release (GtkWidget *widget, GdkEventButton *event)
+button_release (GtkWidget *widget, GdkEventButton *event)
 {
 	const gchar *uri;
 	gboolean keep = FALSE;
 	gint button = event->button;
 	KzBookmarkBaseMenuItem *parent;
-	KzBookmarkMenuItem *item;;
+	KzBookmarkMenuItem *item;
 	GtkAction *action;
 
 	parent = KZ_BOOKMARK_BASE_MENU_ITEM(widget);
@@ -234,7 +210,7 @@ kz_bookmark_menu_item_button_release (GtkWidget *widget, GdkEventButton *event)
 }
 
 static void
-kz_bookmark_menu_item_activate (GtkMenuItem *item)
+activate (GtkMenuItem *item)
 {
 	const gchar *uri;
 	KzBookmarkMenuItem *bitem;
