@@ -20,9 +20,95 @@
  */
 
 #include "kz-embed.h"
+#include "kz-module.h"
 #include "kz-marshalers.h"
 
-#include "gobject-utils.h"
+static GList *embeds = NULL;
+
+static const gchar *
+_kz_embed_module_dir (void)
+{
+	const gchar *base_dir;
+        base_dir = g_getenv("KZ_EMBED_MODULE_DIR");
+	if (base_dir)
+		return base_dir;
+        return KZ_EMBED_MODULEDIR;
+}
+
+void
+kz_embed_load (const gchar *base_dir)
+{
+	if (!base_dir)
+		base_dir = _kz_embed_module_dir();
+
+	embeds = g_list_concat(kz_module_load_modules(base_dir), embeds);
+}
+
+void
+kz_embed_unload (void)
+{
+	g_list_foreach(embeds, (GFunc)kz_module_unload, NULL);
+	g_list_free(embeds);
+	embeds = NULL;
+}
+
+void
+kz_embed_exit (void)
+{
+#warning FIXME: workaround for kz-gecko-embed.cpp.
+	g_list_foreach(embeds, (GFunc)g_type_module_unuse, NULL);
+	g_list_foreach(embeds, (GFunc)kz_module_exit, NULL);
+}
+
+GList *
+kz_embed_engine_names(void)
+{
+	GList *node;
+	GList *result = NULL;
+
+	for (node = embeds; node; node = g_list_next(node))
+	{
+		KzModule *module = node->data;
+		result = g_list_append(result,
+				       g_strdup(kz_module_get_name(module)));
+	}
+
+	return result;
+}
+
+static GtkWidget *
+_kz_embed_new (KzModule *module, const gchar *name,
+	       const gchar *first_property, ...)
+{
+	GObject *embed;
+	va_list var_args;
+
+	va_start(var_args, first_property);
+	embed = kz_module_instantiate(module, first_property, var_args);
+	va_end(var_args);
+
+	return GTK_WIDGET(embed);
+}
+
+GtkWidget *
+kz_embed_new (const gchar *name)
+{
+	KzModule *module;
+
+	module = kz_module_find(embeds, name);
+	if (!module)
+	{
+		module = kz_module_load_module(_kz_embed_module_dir(), name);
+		if (!module)
+			return NULL;
+#warning FIXME: workaround for kz-gecko-embed.cpp.
+		g_type_module_use(G_TYPE_MODULE(module));
+		embeds = g_list_prepend(embeds, module);
+	}
+
+	return _kz_embed_new(module, name, NULL);
+}
+
 
 enum {
 	LINK_MESSAGE_SIGNAL,
@@ -154,7 +240,7 @@ kz_embed_base_init (gpointer g_class)
 				G_SIGNAL_RUN_FIRST,
 				G_STRUCT_OFFSET(KzEmbedIFace, new_window),
 				NULL, NULL,
-				_kz_marshal_VOID__POINTER,
+				g_cclosure_marshal_VOID__POINTER,
 				G_TYPE_NONE, 1, G_TYPE_POINTER);
 
 	kz_embed_signals[OPEN_URI_SIGNAL]
