@@ -247,6 +247,7 @@ struct _KzEmbedIFace {
 	void		(*set_allow_images)	(KzEmbed       *kzembed,
 						 gboolean	allow);
         void            (*show_page_certificate)(KzEmbed       *kzembed);
+        const gchar    *(*get_name)             (KzEmbed       *kzembed);
 
 	/* signals */
 	void     (*link_message)        (KzEmbed      *embed);
@@ -295,6 +296,12 @@ struct _KzEmbedIFace {
 
 GType		kz_embed_get_type		(void) G_GNUC_CONST;
 
+void            kz_embed_load			(const gchar   *base_dir);
+void            kz_embed_unload			(void);
+void            kz_embed_exit			(void);
+GList	       *kz_embed_engine_names		(void);
+
+GtkWidget      *kz_embed_new			(const gchar   *name);
 void		kz_embed_load_url		(KzEmbed       *kzembed,
 						 const gchar   *url);
 void		kz_embed_view_source		(KzEmbed       *kzembed,
