@@ -24,7 +24,6 @@
 #include <gdk/gdkkeysyms.h>
 #include <glib/gi18n.h>
 
-#include "gobject-utils.h"
 #include "kazehakase.h"
 #include "kz-icons.h"
 
@@ -67,13 +66,11 @@ typedef struct _KzPrefsWinPagePriv
 } KzPrefsWinPagePriv; 
 
 
-static void       kz_prefs_win_class_init        (KzPrefsWinClass *klass);
-static void       kz_prefs_win_init              (KzPrefsWin      *win);
-static void       kz_prefs_win_dispose           (GObject         *object);
-static void       kz_prefs_win_response          (GtkDialog       *dialog,
-						  gint             arg);
-static gboolean   kz_prefs_win_delete_event      (GtkWidget       *widget,
-						  GdkEventAny     *event);
+static void       dispose           (GObject         *object);
+static void       response          (GtkDialog       *dialog,
+                                     gint             arg);
+static gboolean   delete_event      (GtkWidget       *widget,
+                                     GdkEventAny     *event);
 
 static GtkWidget *kz_prefs_win_create_nav_tree   (KzPrefsWin      *win);
 static void       kz_prefs_win_reset_nav_tree    (KzPrefsWin      *win);
@@ -95,7 +92,6 @@ static void       cb_tree_cursor_changed         (GtkTreeView     *treeview,
 
 static KzPrefsWin *kz_prefs_win_single = NULL;
 
-static GtkDialogClass *parent_class = NULL;
 static GList *prefs_page_entries = NULL;
 
 
@@ -108,10 +104,7 @@ static GList *prefs_page_entries = NULL;
 						   entry);		\
 }
 
-
-KZ_OBJECT_GET_TYPE(kz_prefs_win, "KzPrefsWin", KzPrefsWin,
-		   kz_prefs_win_class_init, kz_prefs_win_init,
-		   GTK_TYPE_DIALOG)
+G_DEFINE_TYPE(KzPrefsWin, kz_prefs_win, GTK_TYPE_DIALOG)
 
 static void
 kz_prefs_win_class_init (KzPrefsWinClass *klass)
@@ -120,16 +113,15 @@ kz_prefs_win_class_init (KzPrefsWinClass *klass)
 	GtkDialogClass *dialog_class;
 	GtkWidgetClass *widget_class;
 
-	parent_class = g_type_class_peek_parent(klass);
 	object_class = (GObjectClass *) klass;
 	dialog_class = (GtkDialogClass *) klass;
 	widget_class = (GtkWidgetClass *) klass;
 
-	object_class->dispose      = kz_prefs_win_dispose;
+	object_class->dispose      = dispose;
 
-	dialog_class->response     = kz_prefs_win_response;
+	dialog_class->response     = response;
 	
-	widget_class->delete_event = kz_prefs_win_delete_event;
+	widget_class->delete_event = delete_event;
 	
 	ADD_PAGE_ENTRY(prefs_general_get_entry);
 	ADD_PAGE_ENTRY(prefs_lang_get_entry);
@@ -210,19 +202,19 @@ kz_prefs_win_init (KzPrefsWin *win)
 	kz_prefs_win_reset_nav_tree(win);
 
 	/* observe UI Level */
-	g_signal_connect(kz_global_profile,
+	g_signal_connect(KZ_GET_GLOBAL_PROFILE,
 			 "changed::Global",
 			 G_CALLBACK(cb_profile_global_changed), win);
 }
 
 static void
-kz_prefs_win_dispose (GObject *object)
+dispose (GObject *object)
 {
 	KzPrefsWin *win = KZ_PREFS_WIN(object);
 	KzPrefsWinPrivate *priv = KZ_PREFS_WIN_GET_PRIVATE (object);
 
 	g_signal_handlers_disconnect_by_func
-		(kz_global_profile,
+		(KZ_GET_GLOBAL_PROFILE,
 		 G_CALLBACK(cb_profile_global_changed), win);
 
 	if (priv->page_list)
@@ -235,13 +227,13 @@ kz_prefs_win_dispose (GObject *object)
 	if (kz_prefs_win_single && kz_prefs_win_single == win)
 		kz_prefs_win_single = NULL;
 
-	if (G_OBJECT_CLASS(parent_class)->dispose)
-		G_OBJECT_CLASS(parent_class)->dispose(object);
+	if (G_OBJECT_CLASS(kz_prefs_win_parent_class)->dispose)
+		G_OBJECT_CLASS(kz_prefs_win_parent_class)->dispose(object);
 }
 
 
 static void
-kz_prefs_win_response (GtkDialog *dialog, gint arg)
+response (GtkDialog *dialog, gint arg)
 {
 	KzPrefsWinPrivate *priv = KZ_PREFS_WIN_GET_PRIVATE (dialog);
 	GList *node = priv->page_list;
@@ -275,7 +267,7 @@ kz_prefs_win_response (GtkDialog *dialog, gint arg)
 }
 
 static gboolean 
-kz_prefs_win_delete_event (GtkWidget *widget, GdkEventAny *event)
+delete_event (GtkWidget *widget, GdkEventAny *event)
 {
 	kz_prefs_win_store_state(KZ_PREFS_WIN(widget));
 	
@@ -285,15 +277,12 @@ kz_prefs_win_delete_event (GtkWidget *widget, GdkEventAny *event)
 GtkWidget *
 kz_prefs_win_new (void)
 {
-	GObject *obj;
-	KzPrefsWin *win;
+	GtkWidget *win;
 
-	obj = g_object_new(KZ_TYPE_PREFS_WIN,
-			   "title", _("Preference - Kazehakase"),
-			   NULL);
-	win = KZ_PREFS_WIN(obj);
-
-	return GTK_WIDGET(win);
+	win = GTK_WIDGET(g_object_new(KZ_TYPE_PREFS_WIN,
+				      "title", _("Preference - Kazehakase"),
+				      NULL));
+	return win;
 }
 
 GtkWidget *
@@ -481,6 +470,16 @@ kz_prefs_win_ensure_parent (KzPrefsWin *win,
 	return FALSE;
 }
 
+static void
+kz_prefs_win_tree_view_mapping_func(GtkTreeView *tree_view,
+				    GtkTreePath *path,
+				    gpointer user_data)
+{
+	GList **expanded_paths = user_data;
+	*expanded_paths = g_list_prepend(*expanded_paths,
+					 gtk_tree_path_copy(path));
+}
+
 static void
 kz_prefs_win_reset_nav_tree (KzPrefsWin *win)
 {
@@ -488,6 +487,7 @@ kz_prefs_win_reset_nav_tree (KzPrefsWin *win)
 	GtkTreeStore *store;
 	GtkTreePath *path;
 	GList *node;
+	GList *expanded_paths = NULL;
 	KzPrefsWinPrivate *winpriv;
 
 	g_return_if_fail(KZ_IS_PREFS_WIN(win));
@@ -496,6 +496,9 @@ kz_prefs_win_reset_nav_tree (KzPrefsWin *win)
 	treeview = GTK_TREE_VIEW(win->tree_view);
 	store = GTK_TREE_STORE(gtk_tree_view_get_model(treeview));
 	gtk_tree_view_get_cursor(treeview, &path, NULL);
+	gtk_tree_view_map_expanded_rows(treeview,
+					kz_prefs_win_tree_view_mapping_func,
+					(gpointer)&expanded_paths);
 
 	gtk_tree_store_clear(store);
 	for (node = prefs_page_entries; node; node = g_list_next(node))
@@ -522,7 +525,7 @@ kz_prefs_win_reset_nav_tree (KzPrefsWin *win)
 							   priv);
 		}
 
-		if (!(priv->entry->ui_level & kz_ui_level()))
+		if (!(priv->entry->ui_level & KZ_GET_UI_LEVEL))
 		{
 			if (priv->widget)
 				gtk_widget_destroy(priv->widget);
@@ -551,6 +554,15 @@ kz_prefs_win_reset_nav_tree (KzPrefsWin *win)
 		g_free(title);
 	}
 
+	for (node = expanded_paths; node; node = g_list_next(node))
+	{
+		GtkTreePath *expanded_path = (GtkTreePath *)(node->data);
+		gtk_tree_view_expand_to_path(treeview, expanded_path);
+		gtk_tree_path_free(expanded_path);
+		node->data = NULL;
+	}
+	g_list_free(expanded_paths);
+		
 	if (path) {
 		gtk_tree_view_set_cursor(treeview, path, NULL, FALSE);
 		gtk_tree_path_free(path);
@@ -624,7 +636,7 @@ kz_prefs_win_set_ui_level (KzPrefsWin *win)
 {
 	gint response;
 
-	switch (kz_ui_level())
+	switch (KZ_GET_UI_LEVEL)
 	{
 	case KZ_UI_LEVEL_BEGINNER:
 		response = KZ_RESPONSE_UI_LEVEL_BEGINNER;
@@ -639,7 +651,7 @@ kz_prefs_win_set_ui_level (KzPrefsWin *win)
 		response = KZ_RESPONSE_UI_LEVEL_CUSTOM;
 		break;
 	default:
-		g_warning("Invalid UI Level: %d", kz_ui_level());
+		g_warning("Invalid UI Level: %d", KZ_GET_UI_LEVEL);
 		response = KZ_RESPONSE_UI_LEVEL_BEGINNER;
 		break;
 	}
