@@ -23,7 +23,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <glib/gi18n.h>
-#include "gobject-utils.h"
 #include "gtk-utils.h"
 #include "kazehakase.h"
 #include "kz-actions.h"
@@ -31,6 +30,7 @@
 #include "kz-icons.h"
 #include "kz-favicon.h"
 #include "kz-bookmark-file.h"
+#include "kz-notebook.h"
 
 enum {
 	PROP_0,
@@ -58,49 +58,46 @@ static gchar *label_color[KZ_TAB_LABEL_N_STATE];
 #define DEFAULT_AUTO_REFRESH_INTERVAL_SEC 300
 
 /* object class */
-static void     kz_tab_label_class_init         (KzTabLabelClass *klass);
-static void     kz_tab_label_init               (KzTabLabel *kztab);
-static GObject *kz_tab_label_constructor        (GType type,
-						 guint n_props,
-						 GObjectConstructParam *props);
-static void     kz_tab_label_dispose            (GObject *object);
-static void     kz_tab_label_set_property       (GObject *object,
-						 guint prop_id,
-						 const GValue *value,
-						 GParamSpec *pspec);
-static void     kz_tab_label_get_property       (GObject *object,
-						 guint prop_id,
-						 GValue *value,
-						 GParamSpec *pspec);
+static GObject *constructor  (GType type,
+                              guint n_props,
+                              GObjectConstructParam *props);
 
+static void     dispose      (GObject      *object);
+static void     set_property (GObject      *object,
+                              guint         prop_id,
+                              const GValue *value,
+                              GParamSpec   *pspec);
+static void     get_property (GObject      *object,
+                              guint         prop_id,
+                              GValue       *value,
+                              GParamSpec   *pspec);
 /* widget class */
-static void     kz_tab_label_realize            (GtkWidget *widget);
-#if 0
-static void     kz_tab_label_unrealize          (GtkWidget *widget);
-#endif
-static gboolean kz_tab_label_button_press       (GtkWidget *widget,
-						 GdkEventButton *event);
-static gboolean kz_tab_label_motion_notify      (GtkWidget *widget,
-						 GdkEventMotion *event);
-static gboolean kz_tab_label_button_release     (GtkWidget *widget,
-						 GdkEventButton *event);
-static gboolean kz_tab_label_scroll_event       (GtkWidget *widget,
-						 GdkEventScroll *event);
-static gboolean kz_tab_label_drag_motion        (GtkWidget *widget,
-						 GdkDragContext *drag_context,
-						 gint x, gint y,
-						 guint time);
-static void     kz_tab_label_drag_data_get      (GtkWidget        *widget,
-						 GdkDragContext   *context,
-						 GtkSelectionData *seldata,
-						 guint             info,
-						 guint             time);
-static void     kz_tab_label_drag_data_received (GtkWidget *widget,
-						 GdkDragContext *drag_context,
-						 gint x, gint y,
-						 GtkSelectionData *data,
-						 guint info,
-						 guint time);
+static void     realize        (GtkWidget        *widget);
+static gboolean button_press   (GtkWidget        *widget,
+                                GdkEventButton   *event);
+static gboolean motion_notify  (GtkWidget        *widget,
+                                GdkEventMotion   *event);
+static gboolean button_release (GtkWidget        *widget,
+                                GdkEventButton   *event);
+static gboolean scroll_event   (GtkWidget        *widget,
+                                GdkEventScroll   *event);
+static gboolean drag_motion    (GtkWidget        *widget,
+                                GdkDragContext   *drag_context,
+		   		gint              x,
+                                gint              y,
+		   		guint             time);
+static void     drag_data_get  (GtkWidget        *widget,
+                                GdkDragContext   *context,
+                                GtkSelectionData *seldata,
+                                guint             info,
+                                guint             time);
+static void     drag_data_received (GtkWidget        *widget,
+                                    GdkDragContext   *drag_context,
+                                    gint              x, 
+                                    gint              y,
+                                    GtkSelectionData *data,
+                                    guint             info,
+                                    guint             time);
 
 static void     kz_tab_label_sync_to_profile    (KzTabLabel *kztab);
 static void     kz_tab_label_move_page          (KzTabLabel *kztab,
@@ -126,8 +123,6 @@ static void cb_destroy       (GtkObject *object,  KzTabLabel *kztab);
 
 static void make_progress_circle (GtkWidget *widget);
 
-static GtkHBoxClass *parent_class = NULL;
-
 enum {
 	TARGET_KAZEHAKASE_TAB,
 	TARGET_NETSCAPE_URL,
@@ -148,9 +143,7 @@ static guint n_url_drag_types = G_N_ELEMENTS (url_drag_types);
 
 static GdkColor red = { 0, 0xffff, 0x0000, 0x0000 };
 
-KZ_OBJECT_GET_TYPE(kz_tab_label, "KzTabLabel", KzTabLabel,
-		   kz_tab_label_class_init, kz_tab_label_init,
-		   GTK_TYPE_HBOX)
+G_DEFINE_TYPE(KzTabLabel, kz_tab_label, GTK_TYPE_HBOX)
 
 static void
 kz_tab_label_class_init (KzTabLabelClass *klass)
@@ -158,29 +151,24 @@ kz_tab_label_class_init (KzTabLabelClass *klass)
 	GObjectClass *gobject_class;
 	GtkWidgetClass *widget_class;
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	gobject_class = (GObjectClass *) klass;
 	widget_class  = (GtkWidgetClass *) klass;
 
 	/* GtkObject signals */
-	gobject_class->constructor  = kz_tab_label_constructor;
-	gobject_class->dispose      = kz_tab_label_dispose;
-	gobject_class->set_property = kz_tab_label_set_property;
-	gobject_class->get_property = kz_tab_label_get_property;
+	gobject_class->constructor  = constructor;
+	gobject_class->dispose      = dispose;
+	gobject_class->set_property = set_property;
+	gobject_class->get_property = get_property;
 
 	/* GtkWidget signales */
-	widget_class->realize              = kz_tab_label_realize;
-#if 0
-	widget_class->unrealize            = kz_tab_label_unrealize;
-#endif
-	widget_class->button_press_event   = kz_tab_label_button_press;
-	widget_class->scroll_event         = kz_tab_label_scroll_event;
-	widget_class->motion_notify_event  = kz_tab_label_motion_notify;
-	widget_class->button_release_event = kz_tab_label_button_release;
-	widget_class->drag_motion          = kz_tab_label_drag_motion;
-	widget_class->drag_data_get        = kz_tab_label_drag_data_get;
-	widget_class->drag_data_received   = kz_tab_label_drag_data_received;
+	widget_class->realize       = realize;
+	widget_class->button_press_event   = button_press;
+	widget_class->button_release_event = button_release;
+	widget_class->scroll_event         = scroll_event;
+	widget_class->motion_notify_event  = motion_notify;
+	widget_class->drag_motion          = drag_motion;
+	widget_class->drag_data_get        = drag_data_get;
+	widget_class->drag_data_received   = drag_data_received;
 
 	g_object_class_install_property
 		(gobject_class,
@@ -308,7 +296,7 @@ kz_tab_label_init (KzTabLabel *kztab)
                           url_drag_types,n_url_drag_types,
 			  GDK_ACTION_MOVE);
 
-	g_signal_connect(kz_global_profile, "changed::Tab",
+	g_signal_connect(KZ_GET_GLOBAL_PROFILE, "changed::Tab",
 			 G_CALLBACK(cb_profile_changed), kztab);
 	kz_tab_label_sync_to_profile(kztab);
 	kz_tab_label_set_text(kztab, NULL);
@@ -316,13 +304,13 @@ kz_tab_label_init (KzTabLabel *kztab)
 
 
 static GObject*
-kz_tab_label_constructor (GType                  type,
-			  guint                  n_props,
-			  GObjectConstructParam *props)
+constructor (GType                  type,
+             guint                  n_props,
+             GObjectConstructParam *props)
 {
 	KzTabLabel *kztab;
 	GObject *object;
-	GObjectClass *klass = G_OBJECT_CLASS(parent_class);
+	GObjectClass *klass = G_OBJECT_CLASS(kz_tab_label_parent_class);
 	gchar *title;
 
 	object = klass->constructor(type, n_props, props);
@@ -351,7 +339,7 @@ kz_tab_label_constructor (GType                  type,
 
 
 static void
-kz_tab_label_dispose (GObject *object)
+dispose (GObject *object)
 {
 	KzTabLabel *kztab;
 	KzTabLabelPrivate *priv;
@@ -359,7 +347,7 @@ kz_tab_label_dispose (GObject *object)
 	kztab = KZ_TAB_LABEL(object);
 	priv = KZ_TAB_LABEL_GET_PRIVATE (kztab);
 
-	g_signal_handlers_disconnect_by_func(kz_global_profile,
+	g_signal_handlers_disconnect_by_func(KZ_GET_GLOBAL_PROFILE,
 					     G_CALLBACK(cb_profile_changed),
 					     kztab);
 	if (kztab->kzembed)
@@ -383,24 +371,17 @@ kz_tab_label_dispose (GObject *object)
 	/* if connected signal exists, disconnect it. */
 	if (priv->favicon_signal)
 	{
-		g_signal_handler_disconnect(kztab->kz->kzfav,
+		g_signal_handler_disconnect(KZ_GET_FAVICON,
 					    priv->favicon_signal);
 		priv->favicon_signal = 0;
 	}
 
 	if (kztab->history)
 	{
-		gboolean save_session = FALSE;
-		
-		kz_window_append_closed_tab(kztab->kz, kztab->history);
-	
 		kz_bookmark_remove(kztab->kz->tabs, kztab->history);
+		kz_window_append_closed_tab(kztab->kz, kztab->history);
 		g_object_unref(kztab->history);
 		kztab->history = NULL;
-		
-		KZ_CONF_GET("Session", "save", save_session, BOOL);
-		if (save_session && !kztab->kz->is_closing_all)
-			kz_bookmark_file_save(KZ_BOOKMARK_FILE(kz_bookmarks->current_session));
 	}
 
 	if (kztab->kz)
@@ -411,7 +392,7 @@ kz_tab_label_dispose (GObject *object)
 
 	if (kztab->tooltips)
 	{
-		gtk_object_sink(GTK_OBJECT(kztab->tooltips));
+		g_object_ref_sink(GTK_OBJECT(kztab->tooltips));
 		kztab->tooltips = NULL;
 	}
 	
@@ -431,16 +412,16 @@ kz_tab_label_dispose (GObject *object)
 		kztab->mask = NULL;
 	}
 
-	if (G_OBJECT_CLASS (parent_class)->dispose)
-		G_OBJECT_CLASS (parent_class)->dispose(object);
+	if (G_OBJECT_CLASS (kz_tab_label_parent_class)->dispose)
+		G_OBJECT_CLASS (kz_tab_label_parent_class)->dispose(object);
 }
 
 
 static void
-kz_tab_label_set_property (GObject         *object,
-			   guint            prop_id,
-			   const GValue    *value,
-			   GParamSpec      *pspec)
+set_property (GObject         *object,
+              guint            prop_id,
+              const GValue    *value,
+              GParamSpec      *pspec)
 {
 	KzTabLabel *tablabel = KZ_TAB_LABEL(object);
   
@@ -460,10 +441,10 @@ kz_tab_label_set_property (GObject         *object,
 
 
 static void
-kz_tab_label_get_property (GObject         *object,
-			   guint            prop_id,
-			   GValue          *value,
-			   GParamSpec      *pspec)
+get_property (GObject         *object,
+              guint            prop_id,
+              GValue          *value,
+              GParamSpec      *pspec)
 {
 	KzTabLabel *tablabel = KZ_TAB_LABEL(object);
 
@@ -570,10 +551,7 @@ kz_tab_label_set_show_close_button (KzTabLabel *kztab, gboolean show)
 {
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 
-	if (show)
-		gtk_widget_show(kztab->close_button);
-	else
-		gtk_widget_hide(kztab->close_button);
+        g_object_set(kztab->close_button, "visible", show, NULL);
 }
 
 
@@ -582,10 +560,7 @@ kz_tab_label_set_show_lock_button (KzTabLabel *kztab, gboolean show)
 {
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 
-	if (show)
-		gtk_widget_show(kztab->lock_button);
-	else
-		gtk_widget_hide(kztab->lock_button);
+        g_object_set(kztab->lock_button, "visible", show, NULL);
 }
 
 
@@ -594,20 +569,17 @@ kz_tab_label_set_show_favicon (KzTabLabel *kztab, gboolean show)
 {
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 
-	if (show && kztab->favicon_pixbuf)
-		gtk_widget_show(kztab->favicon);
-	else
-		gtk_widget_hide(kztab->favicon);
+        g_object_set(kztab->favicon, "visible", show && kztab->favicon_pixbuf, NULL);
 }
 
 
 static void
-kz_tab_label_realize (GtkWidget *widget)
+realize (GtkWidget *widget)
 {
 	KzTabLabel *kztab = KZ_TAB_LABEL(widget);
 
-	if (GTK_WIDGET_CLASS (parent_class)->realize)
-		GTK_WIDGET_CLASS (parent_class)->realize(widget);
+	if (GTK_WIDGET_CLASS (kz_tab_label_parent_class)->realize)
+		GTK_WIDGET_CLASS (kz_tab_label_parent_class)->realize(widget);
 
 	widget->window = gtk_widget_get_parent_window (widget);
 	g_object_ref (widget->window);
@@ -617,16 +589,9 @@ kz_tab_label_realize (GtkWidget *widget)
 	if (kz_embed_is_loading(kztab->kzembed))
 		kz_tab_label_set_state (kztab, KZ_TAB_LABEL_STATE_LOADING);
 }
-#if 0
-static void
-kz_tab_label_unrealize (GtkWidget *widget)
-{
-	if (GTK_WIDGET_CLASS (parent_class)->unrealize)
-		return GTK_WIDGET_CLASS (parent_class)->unrealize(widget);
-}
-#endif
+
 static gboolean
-kz_tab_label_button_press (GtkWidget *widget, GdkEventButton *event)
+button_press (GtkWidget *widget, GdkEventButton *event)
 {
 	KzTabLabel *kztab = KZ_TAB_LABEL(widget);
 	KzTabLabelPrivate *priv = KZ_TAB_LABEL_GET_PRIVATE (widget);
@@ -654,13 +619,13 @@ kz_tab_label_button_press (GtkWidget *widget, GdkEventButton *event)
 				KZ_EMBED_RELOAD_BYPASS_PROXY_AND_CACHE : 
 				KZ_EMBED_RELOAD_NORMAL);
 	}
-	if (GTK_WIDGET_CLASS(parent_class)->button_press_event)
-		GTK_WIDGET_CLASS(parent_class)->button_press_event(widget, event);
+	if (GTK_WIDGET_CLASS(kz_tab_label_parent_class)->button_press_event)
+		GTK_WIDGET_CLASS(kz_tab_label_parent_class)->button_press_event(widget, event);
 	return FALSE;
 }
 
 static gboolean
-kz_tab_label_motion_notify (GtkWidget *widget, GdkEventMotion *event)
+motion_notify (GtkWidget *widget, GdkEventMotion *event)
 {
 	KzTabLabel *kztab = KZ_TAB_LABEL(widget);
 	KzTabLabelPrivate *priv = KZ_TAB_LABEL_GET_PRIVATE (kztab);
@@ -671,13 +636,13 @@ kz_tab_label_motion_notify (GtkWidget *widget, GdkEventMotion *event)
 		priv->moved = TRUE;
 	}
 
-	if (GTK_WIDGET_CLASS(parent_class)->motion_notify_event)
-		return GTK_WIDGET_CLASS(parent_class)->motion_notify_event(widget, event);
+	if (GTK_WIDGET_CLASS(kz_tab_label_parent_class)->motion_notify_event)
+		return GTK_WIDGET_CLASS(kz_tab_label_parent_class)->motion_notify_event(widget, event);
 	return FALSE;
 }
 
 static gboolean
-kz_tab_label_button_release (GtkWidget *widget, GdkEventButton *event)
+button_release (GtkWidget *widget, GdkEventButton *event)
 {
 	KzTabLabel *kztab = KZ_TAB_LABEL(widget);
 	KzTabLabelPrivate *priv = KZ_TAB_LABEL_GET_PRIVATE (kztab);
@@ -699,51 +664,34 @@ kz_tab_label_button_release (GtkWidget *widget, GdkEventButton *event)
 	priv->start_y = 0;
 	priv->moved = FALSE;
 
-	if (GTK_WIDGET_CLASS(parent_class)->button_release_event)
-		return GTK_WIDGET_CLASS(parent_class)->button_release_event(widget, event);
+	if (GTK_WIDGET_CLASS(kz_tab_label_parent_class)->button_release_event)
+		return GTK_WIDGET_CLASS(kz_tab_label_parent_class)->button_release_event(widget, event);
 	return FALSE;
 }
 
 
 static gboolean
-kz_tab_label_scroll_event (GtkWidget *widget, GdkEventScroll *event)
+scroll_event (GtkWidget *widget, GdkEventScroll *event)
 {
 	KzWindow *kz;
 	gboolean retval = FALSE;
-	GtkAction *action;
 
 	kz = KZ_TAB_LABEL(widget)->kz;
 
-	switch (event->direction) {
-	case GDK_SCROLL_UP:
-	case GDK_SCROLL_LEFT:
-		action = gtk_action_group_get_action(kz->actions, "PrevTab");
-		gtk_action_activate(action);
-		retval = TRUE;
-		break;
-	case GDK_SCROLL_DOWN:
-	case GDK_SCROLL_RIGHT:
-		action = gtk_action_group_get_action(kz->actions, "NextTab");
-		gtk_action_activate(action);
-		retval = TRUE;
-		break;
-	default:
-		g_warning ("Invalid scroll direction!");
-		break;
-	}
+	retval = kz_notebook_scroll_tab(KZ_NOTEBOOK(kz->notebook), event->direction);
 
-	if (GTK_WIDGET_CLASS(parent_class)->scroll_event)
-		return GTK_WIDGET_CLASS(parent_class)->scroll_event(widget, event)
+	if (GTK_WIDGET_CLASS(kz_tab_label_parent_class)->scroll_event)
+		return GTK_WIDGET_CLASS(kz_tab_label_parent_class)->scroll_event(widget, event)
 			|| retval;
 	return retval;
 }
 
 
 static gboolean
-kz_tab_label_drag_motion (GtkWidget *widget,
-			  GdkDragContext *drag_context,
-			  gint x, gint y,
-			  guint time)
+drag_motion (GtkWidget *widget,
+             GdkDragContext *drag_context,
+             gint x, gint y,
+             guint time)
 {
 	KzTabLabel *kztab;
 	KzEmbed *current_kzembed;
@@ -758,19 +706,19 @@ kz_tab_label_drag_motion (GtkWidget *widget,
 	if (KZ_EMBED(kztab->kzembed) == current_kzembed)
 		return FALSE;
 
-	page_num = gtk_notebook_page_num(GTK_NOTEBOOK(kztab->kz->notebook),
-					 GTK_WIDGET(kztab->kzembed));
-	gtk_notebook_set_current_page(GTK_NOTEBOOK(kztab->kz->notebook),
-				      page_num);
+	page_num = kz_notebook_page_num(KZ_NOTEBOOK(kztab->kz->notebook),
+					GTK_WIDGET(kztab->kzembed));
+	kz_notebook_set_current_page(KZ_NOTEBOOK(kztab->kz->notebook), page_num);
+
 	return FALSE;
 }
 
 static void
-kz_tab_label_drag_data_get (GtkWidget        *widget,
-			    GdkDragContext   *context,
-			    GtkSelectionData *data,
-			    guint             info,
-			    guint             time)
+drag_data_get (GtkWidget        *widget,
+               GdkDragContext   *context,
+               GtkSelectionData *data,
+               guint             info,
+               guint             time)
 {
 	KzTabLabel *kztab;
 	const gchar *uri, *title;
@@ -841,12 +789,12 @@ kz_tab_label_drag_data_get (GtkWidget        *widget,
 }
 
 static void
-kz_tab_label_drag_data_received (GtkWidget *widget,
-				 GdkDragContext *drag_context,
-				 gint x, gint y,
-				 GtkSelectionData *data,
-				 guint info,
-				 guint time)
+drag_data_received (GtkWidget *widget,
+                    GdkDragContext *drag_context,
+                    gint x, gint y,
+                    GtkSelectionData *data,
+                    guint info,
+                    guint time)
 {
 	KzTabLabel *kztab;
 
@@ -951,15 +899,15 @@ static void
 kz_tab_label_move_page(KzTabLabel *kztab, KzTabLabel *dest_kztab)
 {
 	GtkWidget *src_page;
-	GtkNotebook *notebook;
+	KzNotebook *notebook;
 	gint dest_pos;
 
 	g_return_if_fail(dest_kztab->kz && dest_kztab->kzembed);
 	g_return_if_fail(kztab->kz && kztab->kzembed);
 
-	notebook = GTK_NOTEBOOK(dest_kztab->kz->notebook);
-	dest_pos = gtk_notebook_page_num(notebook,
-					 GTK_WIDGET(dest_kztab->kzembed));
+	notebook = KZ_NOTEBOOK(dest_kztab->kz->notebook);
+	dest_pos = kz_notebook_page_num(notebook,
+					GTK_WIDGET(dest_kztab->kzembed));
 
 	src_page = GTK_WIDGET(kztab->kzembed);
 	if (dest_kztab->kz != kztab->kz)
@@ -1075,23 +1023,17 @@ cb_net_start (KzEmbed *embed, KzTabLabel *kztab)
 static void
 cb_net_stop (KzEmbed *embed, KzTabLabel *kztab)
 {
-	GtkNotebook *note;
+	KzNotebook *note;
 	gboolean show_favicon;
-	gboolean save_session = FALSE;
 
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 
 	kz_embed_get_history(KZ_EMBED(embed), 
 			     kztab->history);
 
-	/* save session */
-	KZ_CONF_GET("Session", "save", save_session, BOOL);
-	if (save_session && !kztab->kz->is_closing_all)
-		kz_bookmark_file_save(KZ_BOOKMARK_FILE(kz_bookmarks->current_session));
-
-	note = GTK_NOTEBOOK(kztab->kz->notebook);
-	if(gtk_notebook_page_num(note, GTK_WIDGET(embed)) ==
-	   gtk_notebook_get_current_page(note))
+	note = KZ_NOTEBOOK(kztab->kz->notebook);
+	if(kz_notebook_page_num(note, GTK_WIDGET(embed)) ==
+	   kz_notebook_get_current_page(KZ_NOTEBOOK(note)))
 	{
 		kz_tab_label_set_state(kztab, KZ_TAB_LABEL_STATE_NORMAL);
 	}
@@ -1122,7 +1064,7 @@ cb_net_stop (KzEmbed *embed, KzTabLabel *kztab)
 	{
 		const gchar *uri;
 		uri = kz_embed_get_location(KZ_EMBED(kztab->kzembed));
-		kztab->favicon_pixbuf = kz_favicon_get_pixbuf(kztab->kz->kzfav, uri,
+		kztab->favicon_pixbuf = kz_favicon_get_pixbuf(KZ_GET_FAVICON, uri,
 							      GTK_ICON_SIZE_MENU);
 		gtk_image_set_from_pixbuf(GTK_IMAGE(kztab->favicon),
 					  kztab->favicon_pixbuf);
@@ -1212,7 +1154,6 @@ void
 kz_tab_label_set_lock (KzTabLabel *kztab, gboolean lock)
 {
 	KzTabLabelPrivate *priv;
-	gboolean save = FALSE;
 
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 	priv = KZ_TAB_LABEL_GET_PRIVATE (kztab);
@@ -1221,9 +1162,6 @@ kz_tab_label_set_lock (KzTabLabel *kztab, gboolean lock)
 	kz_actions_set_sensitive(kztab->kz, kztab->kzembed);
 	
 	kz_bookmark_set_lock(kztab->history, lock);
-	KZ_CONF_GET("Session", "save", save, BOOL);
-	if (save)
-		kz_bookmark_file_save(KZ_BOOKMARK_FILE(kz_bookmarks->current_session));
 	kz_tab_label_sync_to_profile(kztab);
 }
 
@@ -1242,7 +1180,6 @@ void
 kz_tab_label_set_javascript (KzTabLabel *kztab, gboolean javascript)
 {
 	KzTabLabelPrivate *priv;
-	gboolean save = FALSE;
 
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 	priv = KZ_TAB_LABEL_GET_PRIVATE (kztab);
@@ -1251,9 +1188,6 @@ kz_tab_label_set_javascript (KzTabLabel *kztab, gboolean javascript)
 	kz_actions_set_sensitive(kztab->kz, kztab->kzembed);
 	
 	kz_bookmark_set_javascript(kztab->history, javascript);
-	KZ_CONF_GET("Session", "save", save, BOOL);
-	if (save)
-		kz_bookmark_file_save(KZ_BOOKMARK_FILE(kz_bookmarks->current_session));
 	kz_tab_label_sync_to_profile(kztab);
 }
 
@@ -1294,7 +1228,6 @@ void
 kz_tab_label_set_auto_refresh (KzTabLabel *kztab, gboolean auto_refresh)
 {
 	KzTabLabelPrivate *priv;
-	gboolean save = FALSE;
 
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
 	priv = KZ_TAB_LABEL_GET_PRIVATE (kztab);
@@ -1322,40 +1255,36 @@ kz_tab_label_set_auto_refresh (KzTabLabel *kztab, gboolean auto_refresh)
 	kz_actions_set_sensitive(kztab->kz, kztab->kzembed);
 
 	kz_bookmark_set_auto_refresh(kztab->history, auto_refresh);
-	KZ_CONF_GET("Session", "save", save, BOOL);
-	if (save)
-		kz_bookmark_file_save(KZ_BOOKMARK_FILE(kz_bookmarks->current_session));
 	kz_tab_label_sync_to_profile(kztab);
 }
 
 void
 kz_tab_label_set_history (KzTabLabel *kztab, KzBookmark *history)
 {	
-	GList *children, *node;
-
 	g_return_if_fail(KZ_IS_TAB_LABEL(kztab));
-	
-	kz_bookmark_remove_all(kztab->history);
 
-	children = kz_bookmark_get_children(history);
-	for (node =children; node; node = g_list_next(node))
+	g_object_unref(kztab->history);
+	kztab->history = g_object_ref(history);
+
+	if (kz_bookmark_get_lock(history))
 	{
-		KzBookmark *new;
-		KzBookmark *child = KZ_BOOKMARK(node->data);
-
-		new = kz_bookmark_new_with_attrs(kz_bookmark_get_title(child),
-						 kz_bookmark_get_link(child),
-						 NULL);
-		kz_bookmark_set_last_visited(new, 
-					     kz_bookmark_get_last_visited(child));
-		kz_bookmark_append(kztab->history, new);
-		g_object_unref(new);
+		kz_tab_label_set_lock(kztab, TRUE);
+	}
+	if (kz_bookmark_get_auto_refresh(history))
+	{
+		kz_tab_label_set_auto_refresh(kztab, TRUE);
+	}
+	if (kz_bookmark_get_javascript(history))
+	{
+		kz_embed_set_allow_javascript(KZ_EMBED(kztab->kzembed), TRUE);
+		kz_tab_label_set_javascript(kztab, TRUE);
+	}
+	else
+	{
+		kz_embed_set_allow_javascript(KZ_EMBED(kztab->kzembed), FALSE);
+		kz_tab_label_set_javascript(kztab, FALSE);
 	}
-	kz_bookmark_set_current(kztab->history,
-				kz_bookmark_get_current(history));
 	
-	g_list_free(children);
-
 	kz_embed_set_history(kztab->kzembed, 
 			     kztab->history);
 }
@@ -1376,19 +1305,16 @@ kz_tab_label_set_visited (KzTabLabel *kztab)
 	g_list_free(children);
 
 	if (bookmark)
-	{
-		gboolean save_session = FALSE;
 		kz_bookmark_set_last_visited(bookmark, now.tv_sec);
-		/* save session */
-		KZ_CONF_GET("Session", "save", save_session, BOOL);
-		if (save_session && !kztab->kz->is_closing_all)
-			kz_bookmark_file_save(KZ_BOOKMARK_FILE(kz_bookmarks->current_session));
-	}
+
 	/* find the bookmark which has the same uri from the system bookmark */
 	uri = kz_embed_get_location(kztab->kzembed);
-	bookmark = kz_bookmark_find_bookmark_from_uri(kz_bookmarks->menu,
+	bookmark = kz_bookmark_find_bookmark_from_uri(KZ_GET_MENU_BOOKMARK,
 						      uri);
 	if (bookmark)
 		kz_bookmark_set_last_visited(bookmark, now.tv_sec);
 }
 
+/*
+vim:ts=8:nowrap:ai:expandtab
+*/
