@@ -29,37 +29,13 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
-#ifdef USE_SSL
-#include <gcrypt.h>
-#include <gnutls/gnutls.h> 
-#include <errno.h>
-#include <pthread.h>
-GCRY_THREAD_OPTION_PTHREAD_IMPL;
-#endif
-
 #include "kazehakase.h"
-#include "kz-icons.h"
 #include "kz-window.h"
-#include "kz-actions-tab.h"
-#include "kz-xbel.h"
 #include "kz-ext.h"
-#include "locale.h"
+#include "kz-actions-tab.h"
 #include "utils.h"
 
-#if USE_HYPERESTRAIER
-#	include "estsearch.h"
-#elif USE_RAST
-#	include "rast-search.h"
-#endif
-
-#include "mozilla.h"
-
-KzProfile      *kz_global_profile = NULL;
-KzProfile      *kz_proxy          = NULL;
-KzRootBookmark *kz_bookmarks      = NULL;
-#if USE_MIGEMO
-Migemo         *migemo            = NULL;
-#endif
+KzApp	       *kz_app		  = NULL;
 
 static gchar** action_array;
 static gchar* geometry_string;
@@ -77,80 +53,8 @@ static const gchar parameter_string[] = "URI ...";
 static GOptionContext *context;
 static GOptionGroup *main_group;
 
-gboolean is_restoring_session = FALSE;
-
-gboolean exists_search_cmd = FALSE;
-
 static void activate_action(const gchar *text);
 
-/******************************************************************************
- *                                                                            *
- *                                 UI Level                                   *
- *                                                                            *
- ******************************************************************************/
-KzUILevel
-kz_ui_level (void)
-{
-	KzUILevel val;
-	gchar level[16];
-
-	g_return_val_if_fail(KZ_IS_PROFILE(kz_global_profile),
-			     KZ_UI_LEVEL_BEGINNER);
-
-	level[0] = '\0';
-	kz_profile_get_value(kz_global_profile,
-			     "Global", "ui_level",
-			     level, sizeof(level),
-			     KZ_PROFILE_VALUE_TYPE_STRING);
-
-	val = kz_ui_level_from_str(level);
-	if (!val)
-		return KZ_UI_LEVEL_BEGINNER;
-
-	return val;
-}
-
-
-KzUILevel
-kz_ui_level_from_str (const gchar *level)
-{
-	if (!level || !*level)
-		return KZ_UI_LEVEL_INVALID;
-
-	if (!strcmp(level, "custom"))
-		return KZ_UI_LEVEL_CUSTOM;
-	else if (!strcmp(level, "expert"))
-		return KZ_UI_LEVEL_EXPERT;
-	else if (!strcmp(level, "medium"))
-		return KZ_UI_LEVEL_MEDIUM;
-	else if (!strcmp(level, "beginner"))
-		return KZ_UI_LEVEL_BEGINNER;
-
-	return KZ_UI_LEVEL_INVALID;
-}
-
-
-const gchar *
-kz_ui_level_to_str (KzUILevel level)
-{
-	switch (level)
-	{
-	case KZ_UI_LEVEL_CUSTOM:
-		return "custom";
-	case KZ_UI_LEVEL_EXPERT:
-		return "expert";
-	case KZ_UI_LEVEL_MEDIUM:
-		return "medium";
-	case KZ_UI_LEVEL_BEGINNER:
-		return "beginner";
-	default:
-		break;
-	}
-
-	return NULL;
-}
-
-
 /******************************************************************************
  *                                                                            *
  *                       arguments parser                                     *
@@ -226,12 +130,14 @@ process_args(int argc, char** argv)
 {
 	int i;
 	static int count = 0;
-	GList* list;
+	const GList* list;
 	KzWindow *kz;
 
 #warning This function has side effect!
 
-	list = kz_window_get_window_list();
+	if (!kz_app) return;
+
+	list = KZ_GET_WINDOW_LIST;
 	if (!list || !list->data || !KZ_IS_WINDOW(list->data))
 		return;
 
@@ -316,7 +222,7 @@ activate_action(const gchar *text)
 {
 	KzWindow *kz;
 	KzTabLabel *kztab;
-	GList *window_list;
+	const GList *window_list;
 	GtkWidget *widget;
 	GtkAction *action;
 	gchar *action_name;
@@ -327,7 +233,7 @@ activate_action(const gchar *text)
 	};
 	gint window_id = 0, tab_id = -1 /* -1 is current tab */;
 
-	window_list = kz_window_get_window_list();
+	window_list = KZ_GET_WINDOW_LIST;
 	
 	if (g_str_has_prefix(text, "KzWindowTabPopup"))
 	{
@@ -377,7 +283,7 @@ activate_action(const gchar *text)
 	}
 	window_id = !window_id ? 1 : window_id;
 
-	kz = g_list_nth_data(window_list, window_id - 1);	
+	kz = g_list_nth_data((GList *)window_list, window_id - 1);	
 	/* if not exists indicated window, use first one  */
 	if (!kz)
 		kz = window_list->data;
@@ -503,248 +409,40 @@ check_kazehakase_process (int argc, char *argv[])
 }
 
 
-static void
-clean_history_cache (void)
-{
-	gchar *history_dir;
-	guint limit_days = 30;
-	gboolean limit = FALSE;
-	time_t limit_seconds;
-	
-	KZ_CONF_GET("History", "limit_days", limit_days, INT);
-	KZ_CONF_GET("History", "limit_cache", limit, BOOL);
-	if (limit)
-	{
-		limit_seconds = limit_days * 86400;
-		history_dir = g_strconcat(g_get_home_dir(),
-					  HISTORY_DIR,
-					  NULL); 
-		purge_history_file(history_dir, limit_seconds);
-		g_free(history_dir);
-
-		if (exists_search_cmd)
-		{
-#if USE_HYPERESTRAIER
-			estsearch_purge_index();
-#elif USE_RAST
-#endif
-		}
-	}
-}
-
-static void
-prepare_dir (void)
-{
-	/* for thumbnails */
-	make_thumbnails_dir();
-	
-	/* for favicon */
-	make_dir("favicon");
-
-	/* for popup */
-	make_dir("popup");
-
-	/* for smart bookmark input history */
-	make_dir("smartbookmark_history");
-	
-	/* for storing form data */
-	make_dir("form_data");
-}
-
 /******************************************************************************
  *                                                                            *
  *                                  Main                                      *
  *                                                                            *
  ******************************************************************************/
-static void
-open_app_info (void)
-{
-	gchar *sysconf_file, *conf_file;
-	gchar *bookmark_file, *sys_bookmark_file;
-	gchar *clip_file, *current_session_file;
-	gchar *bookmark_bar_file, *sys_bookmark_bar_file;
-	gchar *smartbookmark_file, *sys_smartbookmark_file;
-	gchar *accel_prefs_file;
-	gchar *proxy_file, *sysproxy_file;
-	gchar *search_command;
-	gboolean exists_index, use_history_search = FALSE;
-
-	/* load prefs */
-	sysconf_file = g_strdup(KZ_SYSCONFDIR "/kzrc");
-	conf_file = g_build_filename(g_get_home_dir(),
-				     "."PACKAGE,
-				     "kzrc", NULL);
-	
-	kz_global_profile = kz_profile_open(conf_file, sysconf_file);
-
-	/* check HyperEstraier Index */
-	KZ_CONF_GET("History", "store_cache", use_history_search, BOOL);
-	if (use_history_search)
-	{
-#if USE_HYPERESTRAIER
-		search_command = g_find_program_in_path("estcmd");
-		exists_index = estsearch_exist_index_dir();
-#elif USE_RAST
-		search_command = g_find_program_in_path("rast");
-		exists_index = rast_exist_index_dir();
-#endif
-		if (search_command)
-		{
-			exists_search_cmd = TRUE;
-			if (!exists_index)
-#if USE_HYPERESTRAIER
-				estsearch_make_index();
-#elif USE_RAST
-				rast_make_index();
-#endif
-			g_free(search_command);
-		}
-	}
-
-	/* load bookmarks */
-	bookmark_file         = g_build_filename(g_get_home_dir(),
-					    	 "."PACKAGE,
-					    	 "bookmarks.xml",
-					    	 NULL);
-	sys_bookmark_file     = g_build_filename(KZ_SYSCONFDIR,
-					    	 "bookmarks.xml",
-					    	 NULL);
-
-        bookmark_bar_file     = g_build_filename(g_get_home_dir(),
-					    	 "."PACKAGE,
-						 "bookmarkbar.xml",
-					    	 NULL);
-	sys_bookmark_bar_file = g_build_filename(KZ_SYSCONFDIR,
-					    	 "bookmarkbar.xml",
-					    	 NULL);
-	clip_file             = g_build_filename(g_get_home_dir(),
-					    	 "."PACKAGE,
-						 "clip.xml",
-					    	 NULL);
-        smartbookmark_file    = g_build_filename(g_get_home_dir(),
-					    	 "."PACKAGE,
-					    	 "smartbookmarks.xml",
-					    	 NULL);
-	sys_smartbookmark_file= g_build_filename(KZ_SYSCONFDIR,
-					    	 "smartbookmarks.xml",
-					    	 NULL);
-	current_session_file  = g_build_filename(g_get_home_dir(),
-						 "."PACKAGE,
-						 "current_session.xml",
-					    	 NULL);
-	
-	kz_bookmarks = kz_root_bookmark_new(bookmark_file, sys_bookmark_file,
-					    clip_file, NULL);
-
-	kz_root_bookmark_add_smartbookmark_file(kz_bookmarks,
-						smartbookmark_file,
-						sys_smartbookmark_file);
-	kz_root_bookmark_add_bookmark_bar_file(kz_bookmarks,
-					       bookmark_bar_file,
-					       sys_bookmark_bar_file);
-	kz_root_bookmark_add_current_session_file(kz_bookmarks,
-						  current_session_file);
-
-	/* Load other prefs... */
-	accel_prefs_file = g_build_filename (g_get_home_dir(),
-					     "."PACKAGE,
-					     "keyaccelrc",
-					     NULL);
-	gtk_accel_map_load (accel_prefs_file);
-
-	sysproxy_file = g_build_filename(KZ_SYSCONFDIR, "proxyrc", NULL);
-	proxy_file = g_build_filename (g_get_home_dir(),
-				       "."PACKAGE,
-				       "proxyrc", NULL);
-	kz_proxy = kz_profile_open(proxy_file, sysproxy_file);
-
-	kz_icons_init();
-	mozilla_init();
-
-#ifdef USE_SSL
-        /* initialize gnutls. this function should be called once. */
-	gcry_control (GCRYCTL_SET_THREAD_CBS, &gcry_threads_pthread);
-	gnutls_global_init ();
-#endif
-
-	kz_ext_init();
-
-	/* make dirs */
-	prepare_dir();
-
-	g_free(sysconf_file);
-	g_free(conf_file);
-	g_free(bookmark_file);
-	g_free(sys_bookmark_file);
-	g_free(clip_file);
-	g_free(current_session_file);
-	g_free(bookmark_bar_file);
-	g_free(sys_bookmark_bar_file);
-	g_free(smartbookmark_file);
-	g_free(sys_smartbookmark_file);
-	g_free(accel_prefs_file);
-	g_free(sysproxy_file);
-	g_free(proxy_file);
-}
-
-
-static void
-close_app_info (void)
-{
-	gchar *accel_prefs_file;
-
-	kz_ext_exit();
-
-	accel_prefs_file = g_build_filename (g_get_home_dir(),
-					     "."PACKAGE,
-					     "keyaccelrc",
-					     NULL);
-
-	gtk_accel_map_save(accel_prefs_file);
-	g_free(accel_prefs_file);
-
-	mozilla_exit();
-
-	/* clean history cache */
-	clean_history_cache();
-
-	kz_root_bookmark_save_all(kz_bookmarks);
-	g_object_unref(G_OBJECT(kz_bookmarks));
-
-	kz_profile_close(kz_global_profile);
-	kz_profile_close(kz_proxy);
-
-#if USE_MIGEMO
-	if (migemo)
-	{
-		migemo_exit();
-		migemo = NULL;
-	}
-#endif	
-	kz_global_profile = NULL;
-	kz_proxy          = NULL;
-	kz_bookmarks      = NULL;
-	
-	free_goption();
-}
-
-
 int
 main (int argc, char *argv[])
 {
-	KzWindow *kz = NULL;
+	int initial_address;
+	GtkWidget *window = NULL;
 	GtkWidget *dupl_check_server = NULL;
 	gchar **argv_copy;
 	gint argc_copy;
 	gboolean retval;
 	GError *error = NULL;
+	gboolean restore = FALSE;
 
         /* initialize */
-	setlocale(LC_ALL, "");
+#ifdef G_OS_WIN32
+	{
+		gchar *locale_dir;
+		locale_dir = g_build_filename(kz_win32_base_path(),
+					      "share", "locale", NULL);
+		bindtextdomain(PACKAGE, locale_dir);
+		g_free(locale_dir);
+	}
+#else
 	bindtextdomain(PACKAGE, LOCALEDIR);
+#endif
 	bind_textdomain_codeset(PACKAGE, "UTF-8");
 	textdomain(PACKAGE);
 
+	if (!g_thread_supported()) g_thread_init(NULL);
+
         /* initialize goption */
 	context = g_option_context_new (parameter_string);
 	g_option_context_add_main_entries (context, entries, NULL);
@@ -786,22 +484,30 @@ main (int argc, char *argv[])
 		return 0;   /* found kazehakase process */
 	}
 
-	/* Load preference */
-	open_app_info();
+ 	kz_app = kz_app_new(argc, argv);
+	kz_app_init_dependencies(kz_app, &initial_address);
 
-	/* create window */	
-	kz = KZ_WINDOW(kz_window_new(NULL));
+	/* create window */
+	KZ_CONF_GET("Session", "restore", restore, BOOL);
+	if (restore)
+	{
+		window = kz_app_restore_session(kz_app);
+		if (!window)
+			window = kz_app_create_new_window(kz_app, NULL);
+	}
+	else
+		window = kz_app_create_new_window(kz_app, NULL);
 
-       	/* 
-	 * gtk_window_pargse_geometry() needs to be called 
+       	/*
+	 * gtk_window_pargse_geometry() needs to be called
 	 * before gtk_widget_show().
-	 */ 
+	 */
 	if(geometry_string)
 	{
-		gtk_window_parse_geometry(GTK_WINDOW(kz),
+		gtk_window_parse_geometry(GTK_WINDOW(window),
 					  geometry_string);
 	}
-	gtk_widget_show(GTK_WIDGET(kz));
+	gtk_widget_show(window);
 
 	/* process arguments */
 	process_args(argc_copy, argv_copy);
@@ -813,11 +519,9 @@ main (int argc, char *argv[])
 	/* exit */
 	gtk_widget_destroy(dupl_check_server);
 	dupl_check_server = NULL;
-#ifdef USE_SSL
-	gnutls_global_deinit ();
-#endif
 
-	close_app_info();
+	kz_app_quit_dependencies(kz_app);
+	g_object_unref(kz_app);
 
 	return 0;
 }
