@@ -20,24 +20,29 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#  include "config.h"
 #endif /* HAVE_CONFIG_H */
 
+#include "gnet.h"
+
 #include <stdlib.h>
 #include <string.h>
 #define __USE_XOPEN
 #include <time.h>
 #include <glib/gi18n.h>
-#include <unistd.h>
+#ifdef HAVE_UNISTD_H
+#  include <unistd.h>
+#endif
 
 #ifdef USE_SSL
 #include <gnutls/gnutls.h>
 #endif
 
-#include "gobject-utils.h"
+#include "kz-missing.h"
+
 #include "kz-http.h"
-#include "gnet.h"
 #include "kazehakase.h"
 #include "kz-profile.h"
 #include "kz-proxy-item.h"
@@ -123,17 +128,15 @@ struct _KzHTTPPrivate
 
 #define KZ_HTTP_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), KZ_TYPE_HTTP, KzHTTPPrivate))
 
-static void kz_http_class_init      (KzHTTPClass *klass);
-static void kz_http_init            (KzHTTP *http);
-static void kz_http_set_property    (GObject *object,
-				     guint prop_id,
-				     const GValue *value,
-				     GParamSpec *pspec);
-static void kz_http_get_property    (GObject *object,
-				     guint prop_id,
-				     GValue *value,
-				     GParamSpec *pspec);
-static void kz_http_dispose         (GObject *object);
+static void set_property    (GObject *object,
+	        	     guint prop_id,
+	        	     const GValue *value,
+	        	     GParamSpec *pspec);
+static void get_property    (GObject *object,
+	        	     guint prop_id,
+	        	     GValue *value,
+	        	     GParamSpec *pspec);
+static void dispose         (GObject *object);
 
 static GIOStatus kz_http_in_header       (KzHTTP *http,
 					  GIOChannel *iochannel);
@@ -193,12 +196,7 @@ kz_http_method_type_get_type (void)
 	return etype;
 }
 
-
-static KzIOClass *parent_class = NULL;
-
-KZ_OBJECT_GET_TYPE(kz_http, "KzHTTP", KzHTTP,
-		   kz_http_class_init, kz_http_init,
-		   KZ_TYPE_IO)
+G_DEFINE_TYPE(KzHTTP, kz_http, KZ_TYPE_IO)
 
 static void
 kz_http_class_init (KzHTTPClass *klass)
@@ -206,13 +204,12 @@ kz_http_class_init (KzHTTPClass *klass)
 	GObjectClass *object_class;
 	KzIOClass *io_class;
 
-	parent_class = g_type_class_peek_parent (klass);
 	object_class = (GObjectClass *) klass;
 	io_class     = (KzIOClass *) klass;
 
-	object_class->dispose      = kz_http_dispose;
-	object_class->set_property = kz_http_set_property;
-	object_class->get_property = kz_http_get_property;
+	object_class->dispose      = dispose;
+	object_class->set_property = set_property;
+	object_class->get_property = get_property;
 	
 	io_class->read_from_io  = kz_http_read_from_io;
 	io_class->io_start      = kz_http_start;
@@ -293,7 +290,7 @@ kz_http_init (KzHTTP *http)
 
 
 void
-kz_http_dispose (GObject *object)
+dispose (GObject *object)
 {
 	KzHTTPPrivate *priv = KZ_HTTP_GET_PRIVATE (object);
 
@@ -355,16 +352,16 @@ kz_http_dispose (GObject *object)
 	priv->post_data        = NULL;
 	priv->auth_param       = NULL;
 
-	if (G_OBJECT_CLASS (parent_class)->dispose)
-		G_OBJECT_CLASS (parent_class)->dispose(object);
+	if (G_OBJECT_CLASS (kz_http_parent_class)->dispose)
+		G_OBJECT_CLASS (kz_http_parent_class)->dispose(object);
 }
 
 
 static void
-kz_http_set_property (GObject *object,
-		      guint prop_id,
-		      const GValue *value,
-		      GParamSpec *pspec)
+set_property (GObject *object,
+              guint prop_id,
+              const GValue *value,
+              GParamSpec *pspec)
 {
 	KzHTTPPrivate *priv = KZ_HTTP_GET_PRIVATE (object);
 
@@ -392,10 +389,10 @@ kz_http_set_property (GObject *object,
 
 
 static void
-kz_http_get_property (GObject *object,
-		      guint prop_id,
-		      GValue *value,
-		      GParamSpec *pspec)
+get_property (GObject *object,
+              guint prop_id,
+              GValue *value,
+              GParamSpec *pspec)
 {
 	KzHTTPPrivate *priv = KZ_HTTP_GET_PRIVATE (object);
 
@@ -424,11 +421,11 @@ KzHTTP *
 kz_http_new (const gchar *uri)
 {
 	KzHTTP *http;
-	GURI *guri;
+	KzURI *guri;
 	gchar *path = NULL, *hostname = NULL;
 	guint port = 80;
 
-	guri = gnet_uri_new(uri);
+	guri = kz_uri_new(uri);
 
 	if (guri)
 	{
@@ -462,7 +459,7 @@ kz_http_new (const gchar *uri)
 #endif
 
 	if (guri)
-		gnet_uri_delete(guri);
+		kz_uri_delete(guri);
 	g_free(path);
 
 	return http;
@@ -818,7 +815,6 @@ kz_http_in_header (KzHTTP *http, GIOChannel *iochannel)
 		 case '5': /* 5xx Server Error  */
 		 default:
 			{
-				g_warning("%s", buffer);
 				iostatus = G_IO_STATUS_ERROR;
 				break;
 			}
@@ -827,7 +823,7 @@ kz_http_in_header (KzHTTP *http, GIOChannel *iochannel)
 	else if (g_ascii_strncasecmp(buffer, "Content-Length:", 15) == 0)
 	{
 		guint size = (guint)strtol(buffer + 15, NULL, 10);
-		g_object_set(G_OBJECT(KZ_IO(http)),
+		g_object_set(G_OBJECT(http),
 			     "file_size", size, NULL);
 	}
 	else if (g_ascii_strncasecmp(buffer, "Transfer-Encoding:", 18) == 0)
@@ -864,7 +860,7 @@ kz_http_in_header (KzHTTP *http, GIOChannel *iochannel)
 		struct tm t;
 		strptime(buffer + 15,
 			 " %a, %d %b %Y %H:%M:%S %z", &t);
-		g_object_set(G_OBJECT(KZ_IO(http)),
+		g_object_set(G_OBJECT(http),
 			     "last_modified", (guint)mktime(&t), NULL);
 	}
 	else if (g_ascii_strncasecmp(buffer, "WWW-Authenticate:", 17) == 0)
@@ -933,7 +929,7 @@ kz_http_in_chunked_body(KzHTTP *http, GIOChannel *iochannel)
 					      &bytes_read);
 		if (iostatus == G_IO_STATUS_NORMAL)
 		{
-			KZ_IO_CLASS (parent_class)->io_to_buffer(KZ_IO(http), bytes_read, buffer);
+			KZ_IO_CLASS (kz_http_parent_class)->io_to_buffer(KZ_IO(http), bytes_read, buffer);
 
 			priv->chunk_size -= bytes_read;
 		}
@@ -977,7 +973,7 @@ kz_http_in_body(KzHTTP *http, GIOChannel *iochannel)
 
 	if (iostatus == G_IO_STATUS_NORMAL)
 	{	
-		KZ_IO_CLASS (parent_class)->io_to_buffer(KZ_IO(http), bytes_read, buffer);
+		KZ_IO_CLASS (kz_http_parent_class)->io_to_buffer(KZ_IO(http), bytes_read, buffer);
 		if (bytes_read == 0)
 			iostatus = G_IO_STATUS_EOF;
 	}
@@ -1042,11 +1038,11 @@ cb_http_connect(GTcpSocket *socket,
 	KzHTTP *http;
 	GIOStatus iostatus = G_IO_STATUS_NORMAL;
 	GIOChannel *iochannel;
-	gchar *command;
+	gchar *command, *tmp_ua;
 	const gchar *method = methods[0];
 	gsize n;
-	gchar *URL;
-	gchar *host_header;
+	gchar *URL, *host_header, *user_agent = NULL;
+	gboolean override = FALSE;
 	KzHTTPPrivate *priv = KZ_HTTP_GET_PRIVATE (data);
 
 	http = KZ_HTTP(data);
@@ -1107,10 +1103,15 @@ cb_http_connect(GTcpSocket *socket,
 				      priv->hostname, priv->port);
 #endif
 
+	KZ_CONF_GET("Global", "override_user_agent", override, BOOL);
+	if (override)
+		user_agent = KZ_CONF_GET_STR("Global", "user_agent");
+	tmp_ua = user_agent ? user_agent : "Kazehakase/"VERSION;
 	/* Send GET command */
 	if (priv->method == KZ_HTTP_METHOD_POST)
 	{
 		gchar *c_len;
+
 		if (priv->post_data)
 			c_len = g_strdup_printf("%d", strlen(priv->post_data));
 		else
@@ -1119,7 +1120,7 @@ cb_http_connect(GTcpSocket *socket,
 		command = g_strconcat(method, " ",
 				      URL, " HTTP/1.1\r\n",
 				      host_header,
-				      "User-Agent: Kazehakase/"VERSION "\r\n",
+				      "User-Agent: ", tmp_ua, "\r\n",
 				      "Content-Type: text/xml\r\n",
 				      "Content-Length: ", c_len, "\r\n",
 				      "Accept-Encoding: gzip,deflate\r\n",
@@ -1135,7 +1136,7 @@ cb_http_connect(GTcpSocket *socket,
 					URL, " HTTP/1.1\r\n",
 					host_header,
 					"Authorization: ", priv->auth_param->string ,"\r\n",
-					"User-Agent: Kazehakase/"VERSION "\r\n",
+				        "User-Agent: ", tmp_ua, "\r\n",
 					"Accept-Encoding: gzip,deflate\r\n",
 					"Connection: close\r\n\r\n", NULL);
 			g_free (priv->auth_param->string);
@@ -1147,12 +1148,15 @@ cb_http_connect(GTcpSocket *socket,
 			command = g_strconcat(method, " ",
 					URL, " HTTP/1.1\r\n",
 					host_header,
-					"User-Agent: Kazehakase/"VERSION "\r\n",
+				        "User-Agent: ", tmp_ua, "\r\n",
 					"Accept-Encoding: gzip,deflate\r\n",
 					"Connection: close\r\n\r\n", NULL);
 		}
 	}
 
+	if (user_agent)
+		g_free(user_agent);
+
 #ifdef USE_SSL
 	if (priv->kz_ssl)
 	{
@@ -1211,7 +1215,7 @@ cb_http_connect(GTcpSocket *socket,
 	}
 
 	/* call io_set_iochannel, start to loading */
-	KZ_IO_CLASS (parent_class)->io_set_channel(KZ_IO(http));
+	KZ_IO_CLASS (kz_http_parent_class)->io_set_channel(KZ_IO(http));
 }
 
 static void
@@ -1221,13 +1225,11 @@ kz_http_start (KzIO *io)
 	gboolean exist, use_proxy;
 	gchar *http_host = NULL;
 	guint http_port;
-	KzHTTP *http;
 	KzProxyItem *item = NULL;
 	KzHTTPPrivate *priv;
 	
 	g_return_if_fail(KZ_IS_HTTP(io));
-	http = KZ_HTTP(io);
-	priv = KZ_HTTP_GET_PRIVATE (http);
+	priv = KZ_HTTP_GET_PRIVATE (io);
 
 	priv->chunked = FALSE;
 	priv->header = TRUE;
@@ -1276,7 +1278,7 @@ kz_http_start (KzIO *io)
 	/* There is a valid proxy setting, so connect with this proxy*/
 	priv->use_proxy = TRUE;
 	gnet_tcp_socket_connect_async(item->http_host, item->http_port,
-	 			      cb_http_connect, http);
+	 			      cb_http_connect, io);
 
 	g_object_unref(G_OBJECT(item));
 	return;
@@ -1285,7 +1287,7 @@ NO_PROXY:
 	http_host = priv->hostname;
 	http_port = priv->port;
 	gnet_tcp_socket_connect_async(http_host, http_port,
-				      cb_http_connect, http);
+				      cb_http_connect, io);
 	return;
 }
 
@@ -1295,7 +1297,7 @@ kz_http_error (KzHTTP *http)
 {
 	g_return_if_fail(KZ_IS_HTTP(http));
 
-	KZ_IO_CLASS (parent_class)->io_error(KZ_IO(http));
+	KZ_IO_CLASS (kz_http_parent_class)->io_error(KZ_IO(http));
 }
 
 
@@ -1528,29 +1530,6 @@ ssl_init (KzSSL *kz_ssl, GIOChannel *channel)
 		gnutls_perror(ret);
 		return;
 	}
-
-#if 0
-	server_cert = SSL_get_peer_certificate(kz_ssl->ssl);
-
-	if (!server_cert)
-		return;
-
-	str = X509_NAME_oneline(X509_get_subject_name(server_cert), 0, 0);
-	g_warning("%s", str);
-	OPENSSL_free(str);
-
-	str = X509_NAME_oneline(X509_get_issuer_name(server_cert), 0, 0);
-	g_warning("%s", str);
-	OPENSSL_free(str);
-
-	X509_free(server_cert);
-	/*
-	result = SSL_get_verify_result(kz_ssl->ssl);
-	if (result == X509_V_OK)
-	{
-	}
-	*/
-#endif
 }
 
 static void
