@@ -25,12 +25,7 @@
 #include <glib/gi18n.h>
 #include "kazehakase.h"
 #include "utils/gtk-utils.h"
-#if USE_HYPERESTRAIER
-#	include "utils/estsearch.h"
-#elif USE_RAST
-#	include "utils/rast-search.h"
-#endif
-
+#include "kz-search.h"
 
 #define DATA_KEY "KzPrefsHistory::info"
 
@@ -42,11 +37,11 @@ static void           prefs_history_response    (GtkWidget *widget,
 
 static KzPrefsWinPageEntry prefs_entry =
 {
-	path:            N_("/History"),
-	priority_hint:   0,
-	ui_level:        ~KZ_UI_LEVEL_BEGINNER,
-	create:          prefs_history_create,
-	response:        prefs_history_response,
+	/* .path          = */ N_("/History"),
+	/* .priority_hint = */ 0,
+	/* .ui_level      = */ ~KZ_UI_LEVEL_BEGINNER,
+	/* .create        = */ prefs_history_create,
+	/* .response      = */ prefs_history_response,
 };
 
 
@@ -61,6 +56,7 @@ typedef struct _KzPrefsHistory
 	GtkWidget     *max_results;
 	GtkWidget     *num_summary;
 	GtkWidget     *except_keyword;
+	GtkWidget     *search_engine;
 	gboolean       changed;
 } KzPrefsHistory;
 
@@ -89,29 +85,17 @@ cb_changed (GtkWidget *widget, KzPrefsHistory *prefsui)
 	prefsui->changed = TRUE;
 }
 
-static void
-cb_optimize_exit (GPid pid, gint status, gpointer data)
-{
-	g_spawn_close_pid(pid);
-
-	gtk_widget_set_sensitive(GTK_WIDGET(data), TRUE);
-}
 
 static void
 cb_optimize_button_clicked (GtkButton *button, KzPrefsHistory *prefsui)
 {
 	GPid pid;
-	
-	gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);
-
-#if USE_HYPERESTRAIER
-	pid = estsearch_optimize_index();
-#elif USE_RAST
-	pid = rast_optimize_index();
-#endif
-	g_child_watch_add(pid,
-			  cb_optimize_exit,
-			  button);
+	if (KZ_GET_SEARCH)
+	{
+		gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);
+		pid = kz_search_optimize_index(KZ_GET_SEARCH);
+		gtk_widget_set_sensitive(GTK_WIDGET(button), TRUE);
+	}
 }
 
 
@@ -123,6 +107,33 @@ prefs_history_destroy (gpointer data)
 	g_free(prefsui);
 }
 
+static void
+set_search_engine (GtkComboBox *combo)
+{
+	gchar *search_engine_name;
+	gint i = 0, active = 0;
+	GList *node;
+	GList *search_engine_ids = NULL;
+
+	search_engine_name = KZ_CONF_GET_STR("History", "search_engine");
+	gtk_combo_box_append_text(GTK_COMBO_BOX(combo), _("None"));
+
+	search_engine_ids = kz_search_engine_ids();
+	for (node = search_engine_ids; node; node = g_list_next(node))
+	{
+		gchar *engine_id = node->data;
+		gtk_combo_box_append_text(combo, engine_id);
+		i++;
+		if (strcmp(engine_id, search_engine_name) == 0)
+			active = i;
+		g_free(engine_id);
+	}
+	g_list_free(search_engine_ids);
+
+	if (search_engine_name)
+		g_free(search_engine_name);
+	gtk_combo_box_set_active(combo, active);
+}
 
 static GtkWidget *
 prefs_history_create (void)
@@ -130,7 +141,7 @@ prefs_history_create (void)
 	KzPrefsHistory *prefsui = g_new0(KzPrefsHistory, 1);
 	GtkWidget *main_vbox, *vbox, *hbox, *frame;
 	GtkWidget *label, *check, *spin, *button;
-	GtkWidget *entry;
+	GtkWidget *entry, *combo;
 	GtkAdjustment *adj;
 	gboolean limit, store_cache = TRUE;
 	gint limit_days, max_results, num_summary;
@@ -148,11 +159,11 @@ prefs_history_create (void)
 
 
 	/*
-	 *  History Cache 
+	 *  Local Cache 
 	 */
 	KZ_CONF_GET("History", "store_cache", store_cache, BOOL);
 	prefsui->store_cache_check = check =
-		gtk_check_button_new_with_label(_("Store history cache for search"));
+		gtk_check_button_new_with_label(_("Store history cache on local disk"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check),
 				     store_cache);
 	g_signal_connect(check, "toggled",
@@ -236,6 +247,21 @@ prefs_history_create (void)
 	gtk_container_add(GTK_CONTAINER(frame), vbox);
 	gtk_widget_show(vbox);
 	
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
+	gtk_container_add(GTK_CONTAINER(vbox), hbox);
+	gtk_widget_show(hbox);
+	label = gtk_label_new_with_mnemonic(_("Search engine name"));
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
+	gtk_widget_show(label);
+
+	prefsui->search_engine = combo = gtk_combo_box_new_text();
+	set_search_engine(GTK_COMBO_BOX(combo));
+	gtk_widget_show(combo);
+	gtk_box_pack_start (GTK_BOX(hbox), combo, TRUE, TRUE, 0);
+	g_signal_connect(combo, "changed",
+			 G_CALLBACK(cb_changed), prefsui);
+
 	hbox = gtk_hbox_new(FALSE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
 	gtk_container_add(GTK_CONTAINER(vbox), hbox);
@@ -348,6 +374,24 @@ prefs_history_response (GtkWidget *widget, gint response)
 		num_summary = gtk_adjustment_get_value(adj);
 		KZ_CONF_SET("History", "num_summary", num_summary, INT);
 
+		if (gtk_combo_box_get_active(GTK_COMBO_BOX(prefsui->search_engine)) == 0)
+		{
+			KZ_SET_SEARCH(NULL);
+			KZ_CONF_SET_STR("History", "search_engine", "");
+		}
+		else
+		{
+			gchar *search_engine;
+			search_engine = gtk_combo_box_get_active_text
+				(GTK_COMBO_BOX(prefsui->search_engine));
+			if (search_engine)
+			{
+				KZ_SET_SEARCH(search_engine);
+				KZ_CONF_SET_STR("History", "search_engine", search_engine);
+				g_free(search_engine);
+			}
+		}
+
 		prefsui->changed = FALSE;
 
 		break;
