@@ -18,73 +18,58 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #include <string.h>
 #include <ctype.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <unistd.h>
 #include <glib/gstdio.h>
 #include <stdlib.h>
+#ifdef HAVE_UNISTD_H
+#  include <unistd.h>
+#endif
 
 #include "utils.h"
 #include "glib-utils.h"
 #include "gtk-utils.h"
 #include "config.h"
 #include "kazehakase.h"
-#include "egg-pixbuf-thumbnail.h"
+#include "kz-search.h"
 
 #define BUFFER_SIZE 1024
 
-void make_dir (const gchar *dirname)
+#define TIME_STAMP_FORMAT "%ld,%s\n"
+
+gboolean
+kz_utils_cp (const gchar *from, const gchar *to)
 {
-	gint  i = 0, ii = 0, ret;
-	gchar *temp, *temp2;
+	FILE *fp;
+	gboolean success = FALSE;
+	gchar *contents;
+	gsize length;
 
-	temp = g_build_filename(g_get_home_dir(), "."PACKAGE, dirname, NULL);
-	temp2 = g_alloca(strlen(temp) + 1);
+	if (!g_file_get_contents(from, &contents, &length, NULL))
+		return FALSE;
 
-	while (temp[i])
+	fp = g_fopen(to, "wb");
+	if (fp)
 	{
-		temp2[ii++] = temp[i];
-		temp2[ii] = '\0';
-		if ((temp[i + 1] == '/' || !temp[i + 1]) &&
-		    !g_file_test(temp2, G_FILE_TEST_IS_DIR))
-		{
-			ret = g_mkdir(temp2, 0711);
+		success = TRUE;
+		if (fwrite(contents, length, 1, fp) != 1)
+			success = FALSE;
+		fclose(fp);
 
-#if 0 /* show dialog? */
-			if (ret != 0)
-			{
-				perror("Faild to create directory");;
-			}
-#endif
-		}
-		i++;
+		if (!success)
+			g_unlink(to);
 	}
 
-	g_free(temp);	
+	g_free(contents);
+	return success;
 }
 
-
-void make_thumbnails_dir (void)
-{
-	gchar *temp;
-	temp = g_strdup_printf("%s/.thumbnails", g_get_home_dir());
-	if (!g_file_test(temp, G_FILE_TEST_IS_DIR))
-	{
-		g_mkdir(temp, 0711);
-	}
-	g_free(temp);	
-	
-	temp = g_strdup_printf("%s/.thumbnails/large", g_get_home_dir());
-	if (!g_file_test(temp, G_FILE_TEST_IS_DIR))
-	{
-		g_mkdir(temp, 0711);
-	}
-	g_free(temp);	
-}
-
-
 gboolean
 key_seems_sequential (const gchar *key, const gchar *prefix)
 {
@@ -122,11 +107,9 @@ gchar *
 remove_tag (const gchar *string, gsize len)
 {
 	GString *work_string;
-	gchar *begin_pos;
 	guint i = 0;
 	
 	work_string = g_string_new(NULL);
-	begin_pos = strstr(string, "<");
 	while (string[i] != '\0' && i < len)
 	{
 		if (string[i] == '<')
@@ -145,6 +128,86 @@ remove_tag (const gchar *string, gsize len)
 }
 
 
+gchar *
+html_to_text (const gchar *html)
+{
+	GString *work_string;
+	guint i=0;
+	
+	work_string = g_string_new(NULL);
+	while (html[i] != '\0')
+	{
+		gboolean ignore = FALSE;
+		switch (html[i])
+		{
+		case '<':
+		{
+			gchar *name = NULL;
+			if (!g_ascii_strncasecmp(html+i+1, "script", 6))
+			{
+				name = "/script";
+				ignore = TRUE;
+			}
+			else if (!g_ascii_strncasecmp(html+i+1, "style", 5)) 
+			{
+				name = "/style"; 
+				ignore = TRUE;
+			}
+			else if (!g_ascii_strncasecmp(html+i+1, "noscript", 8)) 
+			{
+				name = "/noscript"; 
+				ignore = TRUE;
+			}
+			while (html[i] != '>' && html[i] != '\0')
+				i++;
+			if (ignore)
+			{
+				/* ignore until close tag */
+				while (html[i] != '<' && html[i] != '\0' &&
+				       g_ascii_strncasecmp(html+i+1, name, strlen(name)))
+					i++;
+				while (html[i] != '>' && html[i] != '\0')
+					i++;
+			}
+			break;
+		}
+		case '&':
+			if (!strncmp(html+i, "&amp;", 5))
+			{
+				g_string_append_c(work_string, '&');
+				i+=4;
+			}
+			else if (!strncmp(html+i, "&quot;", 6))
+			{
+				g_string_append_c(work_string, '"');
+				i+=5;
+			}
+			else if (!strncmp(html+i, "&lt;", 4))
+			{
+				g_string_append_c(work_string, '<');
+				i+=3;
+			}
+			else if (!strncmp(html+i, "&gt;", 4))
+			{
+				g_string_append_c(work_string, '>');
+				i+=3;
+			}
+			else
+			{
+				g_string_append_c(work_string, html[i]);
+			}
+			break;
+		default:
+			g_string_append_c(work_string, html[i]);
+			break;
+		}
+		i++;
+	}
+	
+	return g_string_free(work_string, FALSE);
+}
+
+
 gchar *
 create_filename_from_uri (const gchar *uri)
 {
@@ -186,33 +249,33 @@ create_filename_with_path_from_uri (const gchar *uri)
 
 	if (g_str_has_suffix(uri, "/"))
 	{
-		filename = g_strconcat(scheme, "/", pos, "_", NULL);
+		filename = g_strconcat(scheme, G_DIR_SEPARATOR_S, pos, "_", NULL);
 	}
 	else
 	{
-		gchar *pos2 = strchr(uri, '?');
-		if (pos2)
+		gchar *query_pos = strchr(uri, '?');
+		if (query_pos)
 		{
-			gchar *string = g_strndup(pos, pos2 - pos);
+			gchar *string = g_strndup(pos, query_pos - pos);
 			filename = g_strconcat(scheme,
-					       "/",
-					       string, 
-					       "_/",
-					       pos2 + 1,
+					       G_DIR_SEPARATOR_S,
+					       string,
+					       "_"G_DIR_SEPARATOR_S,
+					       query_pos + 1,
 					       NULL);
 			g_free(string);
 		}
 		else
 		{
 			filename = g_strconcat(scheme,
-						"/",
-						pos,
-						NULL);
+					       G_DIR_SEPARATOR_S,
+					       pos,
+					       NULL);
 		}
 	}
 
 	g_free(scheme);
-	
+
 	return filename;
 }
 
@@ -225,28 +288,27 @@ create_uri_from_filename (const gchar *filename)
 	gchar *scheme;
 	gchar *path;
 	
-	pos = strstr(filename, "/");
+	pos = strstr(filename, G_DIR_SEPARATOR_S);
 	
 	if (!pos)
-		return NULL;
+		return g_strdup(filename);
 
 	scheme = g_strndup(filename, pos - filename);
 
-	if (g_str_has_suffix(filename, "/_"))
+	if (g_str_has_suffix(filename, G_DIR_SEPARATOR_S"_"))
 	{
-		path = g_strndup(pos + 1, strlen(pos) - 2); 
+		path = g_strndup(pos + 1, strlen(pos) - 2);
 	}
 	else
 	{
-		gchar *pos2 = g_strrstr(filename, "_/");
-		if (pos2)
+		gchar *query_pos = g_strrstr(filename, "_"G_DIR_SEPARATOR_S);
+		if (query_pos)
 		{
-			gchar *string = g_strndup(pos + 1, pos2 - pos - 1);
-			path = g_strconcat(string, 
+			gchar *string = g_strndup(pos + 1, query_pos - pos - 1);
+			path = g_strconcat(string,
 					   "?",
-					   pos2 + 2,
+					   query_pos + 2,
 					   NULL);
-			
 			g_free(string);
 		}
 		else
@@ -263,7 +325,7 @@ create_uri_from_filename (const gchar *filename)
 	{
 		uri = g_strconcat(scheme, "://", path, NULL);
 	}
-	
+
 	g_free(scheme);
 	g_free(path);
 
@@ -457,25 +519,83 @@ xml_get_attr (const gchar *buffer, const gchar *attr_name)
 	return attr;
 }
 
+static void
+_append_time_stamp(const gchar *path, FILE *time_stamp)
+{
+	struct stat st;
+
+	if (g_stat(path, &st) == 0)
+		g_fprintf(time_stamp, TIME_STAMP_FORMAT, st.st_mtime, path);
+}
 
 void
-purge_history_file(const gchar *path, time_t limit_seconds)
+kz_utils_append_time_stamp(const gchar *target_file,
+			   const gchar *time_stamp_path)
+{
+	FILE *fp;
+
+	fp = g_fopen(time_stamp_path, "a");
+	if (!fp)
+		return;
+
+	_append_time_stamp(target_file, fp);
+	fclose(fp);
+}
+
+static void
+_make_time_stamp (const gchar *dir, FILE *time_stamp)
 {
 	GDir *gdir;
-	const gchar *dir;
-
-	gdir = g_dir_open(path, 0, NULL);
+	const gchar *entry;
 
+	gdir = g_dir_open(dir, 0, NULL);
 	if (!gdir) return;
 
-	while ((dir = g_dir_read_name(gdir)))
+	while ((entry = g_dir_read_name(gdir)))
 	{
-		gchar *dirname;
-		dirname = g_build_filename(path, dir, NULL);
+		gchar *file_name;
+		file_name = g_build_filename(dir, entry, NULL);
 
-		if (g_file_test(dirname, G_FILE_TEST_IS_DIR))
+		if (g_file_test(file_name, G_FILE_TEST_IS_DIR))
+			_make_time_stamp(file_name, time_stamp);
+		else
+			_append_time_stamp(file_name, time_stamp);
+		g_free(file_name);
+	}
+	g_dir_close(gdir);
+}
+
+void
+kz_utils_make_time_stamp(const gchar *target_dir, const gchar *time_stamp_path)
+{
+	FILE *fp;
+
+	fp = g_fopen(time_stamp_path, "w");
+	if (!fp)
+		return;
+
+	_make_time_stamp(target_dir, fp);
+	fclose(fp);
+}
+
+void
+kz_utils_purge_files(const gchar *target_dir, time_t limit_seconds)
+{
+	GDir *dir;
+	const gchar *entry;
+
+	dir = g_dir_open(target_dir, 0, NULL);
+
+	if (!dir) return;
+
+	while ((entry = g_dir_read_name(dir)))
+	{
+		gchar *file_name;
+		file_name = g_build_filename(target_dir, entry, NULL);
+
+		if (g_file_test(file_name, G_FILE_TEST_IS_DIR))
 		{
-			purge_history_file(dirname, limit_seconds);
+			kz_utils_purge_files(file_name, limit_seconds);
 		}
 		else
 		{
@@ -486,74 +606,112 @@ purge_history_file(const gchar *path, time_t limit_seconds)
 			g_get_current_time(&now);
 			t = (time_t)now.tv_sec;
 
-			if (g_stat(dirname, &st) == 0 &&
+			if (g_stat(file_name, &st) == 0 &&
 			    st.st_mtime < t - limit_seconds)
-				g_unlink(dirname);
+				g_unlink(file_name);
 		}
-		g_free(dirname);
+		g_free(file_name);
 	}
-	g_dir_close(gdir);
+	g_dir_close(dir);
 }
 
 void
-purge_history_file_by_timestamp(const gchar *timestamp_path, time_t limit_seconds)
+kz_utils_purge_files_by_time_stamp(const gchar *target_dir,
+				   const gchar *time_stamp_path,
+				   time_t limit_seconds)
 {
 	gchar buf[BUFFER_SIZE];
 	gchar *tmp_file;
-	gint fd;
-	gint i;
+	gint fd, target_dir_len;
 	FILE *fp;
 	GTimeVal now;
 	time_t t;
+	KzSearch *search;
+
+	fp = g_fopen(time_stamp_path, "r");
+	if (!fp)
+	{
+		g_unlink(time_stamp_path);
+		return;
+	}
 
-	g_get_current_time(&now);
-	t = now.tv_sec;
 
 	fd = g_file_open_tmp("kzXXXXXX", &tmp_file, NULL);
+	if (fd == -1)
+		return;
 
-	fp = g_fopen(timestamp_path, "a+");
-	if(fp)
+	search = KZ_GET_SEARCH;
+	g_get_current_time(&now);
+	t = now.tv_sec;
+	target_dir_len = strlen(target_dir);
+	while (fgets(buf, sizeof(buf), fp))
 	{
-		while(fgets(buf, sizeof(buf), fp))
+		gchar *pos, *time, *path, *path_end, *dir_name;
+		gint time_int;
+		struct stat st;
+		gboolean path_exists;
+
+		pos = strstr(buf, ",");
+		time = g_strndup(buf, pos - buf);
+		path = g_strdup(pos+1);
+
+		path_end = strstr(path, "\n");
+		if (path_end)
+			path[path_end - path] = '\0';
+
+		time_int = atoi(time);
+		if (t - time_int < limit_seconds)
 		{
-			gchar *pos;
-			gchar *time , *path;
-			gint time_int;
-			struct stat st;
-
-			pos = g_strrstr(buf, ",");
-			time = g_strndup(buf, pos - buf);
-			path = g_strdup(pos+1);
-
-			for(i = 0;i <= strlen(path);i++)
-				if(path[i] == '\n')
-					path[i] = '\0';
-
-			time_int = atoi(time);
-
-			if(t - time_int > limit_seconds)
-			{
-				if (g_stat(path, &st) == 0 &&
-						t - st.st_mtime > limit_seconds)
-				{
-					/* remove file over storege period */
-					g_unlink(path);
-					g_free(time);
-					g_free(path);
-					continue;
-				}
-			}
-
 			write(fd, buf, strlen(buf));
-
-			g_free(time);
-			g_free(path);
+			goto CLEANUP;
 		}
+
+		path_exists = g_stat(path, &st) == 0;
+		if (!path_exists)
+			goto CLEANUP;
+
+		if (t - st.st_mtime < limit_seconds)
+		{
+			gchar *entry;
+
+			entry = g_strdup_printf(TIME_STAMP_FORMAT,
+						st.st_mtime, path);
+			write(fd, entry, strlen(entry));
+			goto CLEANUP;
+		}
+
+
+		if (search && strlen(path) > target_dir_len)
+		{
+			gchar *uri;
+			gchar *relative_path;
+
+			relative_path = path + target_dir_len;
+			uri = create_uri_from_filename(relative_path);
+			kz_search_unregister_document(search, uri);
+			g_free(uri);
+		}
+		/* remove file over storage period */
+		g_unlink(path);
+
+		dir_name = g_path_get_dirname(path);
+		g_rmdir(dir_name); /* ignore error */
+		g_free(dir_name);
+
+	  CLEANUP:
+		g_free(time);
+		g_free(path);
 	}
+	close(fd);
+	fclose(fp);
+
+	g_unlink(time_stamp_path);
+	kz_utils_cp(tmp_file, time_stamp_path);
+	g_unlink(tmp_file);
 }
 
 gchar *
-ensure_encode_string(const gchar *text, 
+ensure_encode_string(const gchar *text,
 		     const gchar *encode,
 		     gboolean urlencode)
 {
@@ -625,10 +783,10 @@ history_get_last_modified (const gchar *uri)
 	int ret;
 	
 	filename = create_filename_with_path_from_uri(uri);
-	history_filename = g_strconcat(g_get_home_dir(),
-				       HISTORY_DIR,
-				       filename,
-			  	       NULL);
+	history_filename = g_build_filename(g_get_home_dir(),
+					    HISTORY_DIR,
+					    filename,
+					    NULL);
 	
 	ret = g_stat(history_filename, &st);
 	g_free(history_filename);
@@ -640,19 +798,23 @@ history_get_last_modified (const gchar *uri)
 		return 0;
 }
 
+#ifndef MOZ_NSICANVASRENDERINGCONTEXTINTERNAL_HAVE_GETINPUTSTREAM_
 typedef struct _KzPixbufConverter
 {
 	GdkPixbufLoader *loader;
+	gchar *uri;
 	gchar *filename;
+	time_t mtime;
+	EggPixbufThumbSize size;
 } KzPixbufConverter;
 
 
-#ifndef MOZ_NSICANVASRENDERINGCONTEXTINTERNAL_HAVE_GETINPUTSTREAM_
 static gboolean
 idle_dispose_converter(gpointer data)
 {
 	KzPixbufConverter *converter = data;
 
+	g_free(converter->uri);
 	g_free(converter->filename);
 	g_free(converter);
 
@@ -667,7 +829,7 @@ cb_io_in(GIOChannel *iochannel, GIOCondition condition,
 	gsize bytes_read;
 	gchar buffer[BUFFER_SIZE];
 	KzPixbufConverter *converter = data;
-	
+
 	if (condition & G_IO_ERR)
 	{
 		g_idle_add(idle_dispose_converter, converter);
@@ -675,7 +837,7 @@ cb_io_in(GIOChannel *iochannel, GIOCondition condition,
 	}
 
 	/* Read the data into our buffer */
-	iostatus = g_io_channel_read_chars(iochannel, buffer, 
+	iostatus = g_io_channel_read_chars(iochannel, buffer,
 					   sizeof(buffer),
 					   &bytes_read,
 					   NULL);
@@ -690,18 +852,19 @@ cb_io_in(GIOChannel *iochannel, GIOCondition condition,
 	switch (iostatus)
 	{
 	 case G_IO_STATUS_EOF:
-	 {	
+	 {
 		GdkPixbuf *pixbuf;
 		pixbuf = gdk_pixbuf_loader_get_pixbuf(converter->loader);
 		if (pixbuf)
 		{
 			GdkPixbuf *rotate, *scale;
-		
+
 			/* scale */
-			scale = gdk_pixbuf_scale_simple(pixbuf, 
-						        KZ_THUMB_HEIGHT, EGG_PIXBUF_THUMB_LARGE,
+			scale = gdk_pixbuf_scale_simple(pixbuf,
+						        KZ_THUMB_HEIGHT,
+							converter->size,
 						        GDK_INTERP_HYPER);
-		
+
 			/* rotate image */
 			rotate = gdk_pixbuf_rotate_simple(scale,
         						  GDK_PIXBUF_ROTATE_CLOCKWISE);
@@ -709,19 +872,18 @@ cb_io_in(GIOChannel *iochannel, GIOCondition condition,
 			/* save image */
 			if (rotate)
 			{
-				gchar *png_file;
-				png_file = g_strndup(converter->filename,
-						     strlen(converter->filename) - 3);
-				gdk_pixbuf_save(rotate, 
-						png_file, "png", NULL, NULL);
+				egg_pixbuf_add_thumbnail_data(rotate,
+							      converter->uri,
+							      converter->mtime,
+							      converter->size);
+				egg_pixbuf_save_thumbnail(rotate, NULL, NULL);
 				g_object_unref(rotate);
-				g_free(png_file);
 			}
 		}
-		gdk_pixbuf_loader_close(converter->loader, NULL);	
+		gdk_pixbuf_loader_close(converter->loader, NULL);
 		g_unlink(converter->filename);
 		g_idle_add(idle_dispose_converter, converter);
-		
+
 		return FALSE;
   	 }
 	 case G_IO_STATUS_NORMAL:
@@ -734,9 +896,11 @@ cb_io_in(GIOChannel *iochannel, GIOCondition condition,
 
 
 void
-convert_image (const gchar *ps_file, const gchar *png_file)
+kz_utils_save_ps_as_thumbnail (const gchar *ps_file, const gchar *uri,
+			       time_t mtime, EggPixbufThumbSize size)
 {
-	const gchar *gs = "gs -q -dNOPAUSE -dBATCH -r24 -sDEVICE=ppmraw -sOutputFile=-  %s";
+	const gchar *gs =
+		"gs -q -dNOPAUSE -dBATCH -r24 -sDEVICE=ppmraw -sOutputFile=- %s";
 	gchar *command;
 	gint argc, out;
 	gchar **argv = NULL;
@@ -746,7 +910,7 @@ convert_image (const gchar *ps_file, const gchar *png_file)
 	GdkPixbufLoader *loader;
 	KzPixbufConverter *converter;
 
-	command = g_strdup_printf(gs, ps_file);	
+	command = g_strdup_printf(gs, ps_file);
 
 	g_shell_parse_argv(command,
 			   &argc,
@@ -754,7 +918,7 @@ convert_image (const gchar *ps_file, const gchar *png_file)
 			   NULL);
 
         /* Check whether ghostscript is installed or not */
-	if(g_find_program_in_path (argv[0]) == NULL)
+	if (g_find_program_in_path(argv[0]) == NULL)
 	{
 		g_warning("Can't find %s in your PATH.\n"
 			  "Currently kazehakase using gecko engine needs ghostscript to create thumbnails.\n", argv[0]);
@@ -782,8 +946,11 @@ convert_image (const gchar *ps_file, const gchar *png_file)
 	loader = gdk_pixbuf_loader_new_with_type("pnm", NULL);
 	converter = g_new0(KzPixbufConverter, 1);
 	converter->loader = loader;
+	converter->uri = g_strdup(uri);
 	converter->filename = g_strdup(ps_file);
-	
+	converter->mtime = mtime;
+	converter->size = size;
+
 	g_io_add_watch(io,
 		       G_IO_IN | G_IO_PRI | G_IO_ERR | G_IO_HUP,
 		       cb_io_in, converter);
@@ -856,3 +1023,55 @@ find_file (const gchar *path,
 	find_file_recursive (path, fname, &ret, 0, maxdepth);
 	return ret;
 }
+
+void
+kz_str_replace_char (gchar *str, gchar replace_char, gchar new_char)
+{
+	gint i = 0;
+
+	while (str[i] != '\0')
+	{
+		if (str[i] == replace_char)
+			str[i] = new_char;
+		i++;
+	}
+}
+
+gchar *
+kz_uri_encode_last_component (const gchar *uri)
+{
+	gchar *last_component_encoded_uri, *last_sep_pos;
+
+	last_sep_pos = g_strrstr(uri, "/");
+	if (last_sep_pos)
+	{
+		gchar *pre_component, *last_component;
+
+		pre_component = g_strndup(uri, last_sep_pos - uri + 1);
+		last_component = url_encode(last_sep_pos + 1);
+		last_component_encoded_uri =
+			g_strconcat(pre_component, last_component, NULL);
+		g_free(pre_component);
+	}
+	else
+	{
+		last_component_encoded_uri = g_strdup(uri);
+	}
+
+	return last_component_encoded_uri;
+}
+
+#ifdef G_OS_WIN32
+static gchar *win32_base_path = NULL;
+
+const gchar *
+kz_win32_base_path (void)
+{
+	if (win32_base_path)
+		return win32_base_path;
+
+	win32_base_path = g_win32_get_package_installation_directory(PACKAGE,
+								     NULL);
+	return win32_base_path;
+}
+#endif
