@@ -24,13 +24,17 @@
 #include <gtk/gtk.h>
 #include <time.h>
 
+#include "egg-pixbuf-thumbnail.h"
+
 G_BEGIN_DECLS
 
-void     make_dir                    (const gchar *dirname);
-void     make_thumbnails_dir         (void);
+gboolean kz_utils_cp			(const gchar *from,
+					 const gchar *to);
+
 gboolean key_seems_sequential        (const gchar *key,
 				      const gchar *prefix);
 gchar   *remove_tag                  (const gchar *string, gsize len);
+gchar   *html_to_text                (const gchar *string);
 
 gchar   *create_filename_from_uri    (const gchar *uri);
 
@@ -54,15 +58,33 @@ gchar   *complement_scheme           (const gchar *url);
 gchar   *xml_get_content	     (const gchar *buffer);
 gchar   *xml_get_attr		     (const gchar *buffer, const gchar *attr_name);
 
-GSList  *purge_history_file	     (const gchar *path,
-				      time_t limit_seconds);
-void	 convert_image		     (const gchar *ps_file,
-				      const gchar *png_file);
+void     kz_utils_make_time_stamp	(const gchar	*target_dir,
+					 const gchar	*time_stamp_path);
+void     kz_utils_append_time_stamp	(const gchar	*target_file,
+					 const gchar	*time_stamp_path);
+void     kz_utils_purge_files		(const gchar	*path,
+					 time_t		 limit_seconds);
+void     kz_utils_purge_files_by_time_stamp	(const gchar	*target_dir,
+						 const gchar	*time_stamp_path,
+						 time_t		 limit_seconds);
+void	 kz_utils_save_ps_as_thumbnail		(const gchar *ps_file,
+						 const gchar *uri,
+						 time_t mtime,
+						 EggPixbufThumbSize size);
 gboolean str_isdigit 		     (const gchar *str);
 
 GSList  *find_file                   (const gchar *path,
 				      const gchar *fname,
 				      gint maxdepth);
+void	 kz_str_replace_char         (gchar *str,
+				      gchar replace_char,
+				      gchar new_char);
+
+gchar	*kz_uri_encode_last_component	(const gchar *uri);
+
+#ifdef G_OS_WIN32
+const gchar	*kz_win32_base_path	(void);
+#endif
 
 G_END_DECLS
 
